#========================================================================================
# package      DE_EPAGES::GoogleBase::API::CSV::GoogleBaseProductsExportDriver
# state        private
#----------------------------------------------------------------------------------------
# description  This module implements a csv driver to export object GoogleBase products
#
# example      use DE_EPAGES::GoogleBase::API::CSV::GoogleBaseProductsExportDriver;
#
#               my $Driver = DE_EPAGES::GoogleBase::API::CSV::GoogleBaseProductsExportDriver->new(
#                   Handler => $Handler
#               );
#               $Driver->exportCSV;
#========================================================================================
package DE_EPAGES::GoogleBase::API::CSV::GoogleBaseProductsExportDriver;
use base qw ( DE_EPAGES::ProductPortal::API::CSV::PortalExportDriver Exporter);

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::PerlTools qw ( fcmp isValid );
use DE_EPAGES::Core::API::Url qw ( UrlEncodeValue );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::GoogleBase::API::Constants qw (
    $ALLOWED_AMOUNTS
    $ALLOWED_EXCEPTION_ML_VALUES
    $ALLOWED_EXCEPTIONS_KG_VALUES
    $GOOGLE_VALID_UNIT_TYPES
    $GOOGLE_VALID_UNIT_CODES
    $SUPPORTED_LOCAL_IDS
    GOOGLE_DECIMAL_DIGITS
    GOOGLE_DECIMAL_FILL
    $GOOGLE_VALID_AREA_UNIT
    $GOOGLE_VALID_LENGTH_UNIT
    $GOOGLE_VALID_VOLUME_UNIT
    $GOOGLE_VALID_WEIGHT_UNIT
);
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Product::API::Product qw ( GetUnityPrettyUrl );
use DE_EPAGES::Shop::API::Constants qw ( STOCK_IS_ORDERABLE );
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);
use Scalar::Util qw ( blessed );
use Encode qw (decode_utf8);

use constant MAX_LENGTH_TITLE                  => 150;
use constant MAX_LENGTH_DESCRIPTION            => 10000;
use constant GENDER_ATTRIBUTE => 'Gender';
use constant AGE_GROUP_ATTRIBUTE => 'Age_group';
use constant COLOR_ATTRIBUTE => 'Color';
use constant SIZE_ATTRIBUTE => 'Size';
use constant REFERENCE_UNIT => 'RefUnit';
use constant REFERENCE_AMOUNT => 'RefAmount';
use constant MATERIAL_ATTRIBUTE => 'Material';
use constant PATTERN_ATTRIBUTE => 'Pattern';
our @ACCEPTED_WEIGHT_UNITS = qw (pound ounce gram kilogram);

#========================================================================================
my @MAP_GOOGLEBASE_ATTRIBUTES = qw (
    id
    title
    description
    google_product_category
    product_type
    link
    image_link
    condition
    availability
    price
    brand
    gtin
    mpn
    identifier_exists
    tax
    unit_pricing_measure
    unit_pricing_base_measure
    shipping
    shipping_weight
    gender
    age_group
    size
    color
    item_group_id
    material
    pattern
    is_bundle
);

my $MaxLengthTitle = MAX_LENGTH_TITLE;
#========================================================================================
sub ErrorDefinition {
    return {
        'ValueTooLong'            => 'attribute ( #AttributeName ) for product ( #ProductID ) is too long (Maximum #MaxLength characters). Value has been cut.',
        'ProductNoDescription'    => 'product ( #ProductID ) has no description, can not export to GoogleBase.',
        'NotEnoughValuesSet'      => 'product ( #ProductID ) has not set at least two of the fields Manufacturer, UPCEAN, ManufacturerSKU.',
        'ProductNoGoogleCategory' => 'product ( #ProductID ) does not have the attribute #AttributeAlias set',
        'ProductNoGender' =>         'product ( #ProductID ) has no gender.',
        'ProductNoAgeGroup' =>       'product ( #ProductID ) has no age group.',
        'ProductNoColor' =>          'product ( #ProductID ) has no color.',
        'ProductNoSize' =>           'product ( #ProductID ) has no size.',
        'ProductReferenceUnitMisconfigured' => 'product ( #ProductID ) has reference unit misconfigurated.',
        'ProductReferenceAmountMisconfigured' => 'product ( #ProductID ) has reference amount misconfigurated.',
        'ProductNoMaterial' =>       'product ( #ProductID ) has no material attribute.',
        'ProductNoPattern' =>        'product ( #ProductID ) has no pattern attribute.'
    }
}

#========================================================================================
sub exportCSV {
    my $self = shift;
    my ($FileName, $hFileFormat, $aColumnNames, $ViewObject, $aObjects, $PortalConfig) = @_;

    $self->{'PortalLanguageID'} = $PortalConfig->get('LanguageID');

    my $PortalSiteCountry = $ViewObject->get('PortalSite')->get('Country');
    $self->{'PortalSiteUppercaseCountry'} = uc( $PortalSiteCountry->{'Code2'} );
    $self->{'PortalShippingMethodCountries'} = $self->_getShippingMethodCountries($ViewObject->get('ShippingMethod'));

    $hFileFormat = {
        'sep_char'     => "\t",
        'always_quote' => 0,
        'encoding'     => 'utf8'
    } unless defined $hFileFormat;
    $hFileFormat->{'quote_char'} = undef; # EPG-19210: never quote anything
    $ViewObject->get('MaxTitleLength') > MAX_LENGTH_TITLE ? $MaxLengthTitle = MAX_LENGTH_TITLE : $MaxLengthTitle = $ViewObject->get('MaxTitleLength');

    $self->SUPER::exportCSV($FileName, $hFileFormat, [@MAP_GOOGLEBASE_ATTRIBUTES], $ViewObject, $aObjects);
    return;
}

#========================================================================================
sub getCountryCode {
    my $self = shift;
    return lc($self->portalSite->get('Country')->{'Code2'});
}

#========================================================================================
sub isProductValidPortalExt1 {
    my $self = shift;
    my ($Product) = @_;

    $self->SUPER::isProductValidPortalExt1($Product);

    # test for description
    my $DescriptionAttribute = $self->descriptionAttribute;
    if ( defined $DescriptionAttribute ) {
        my $Alias = $DescriptionAttribute->alias;
        Error('ProductNoDescription', {'ProductID'=>$Product->id, 'Product'=>$Product}) unless defined $Product->getWithParent($Alias, $self->languageID );
    }

    return;
}

#========================================================================================
sub isProductValidPortal {
    my $self = shift;
    my ($Product) = @_;

    $self->isProductValidPortalExt1($Product);
    $self->_isReferenceAttributesValid ($Product);

    return;
}

#========================================================================================
sub formatPrice {
    my $self = shift;
    my ($Price) = @_;

    my $CurrencyID = $self->currencyID;
    my $FormatAttributes = $self->portalSite->formatAttributes($CurrencyID);
    my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new(
        decimal_point  => '.',
        thousands_sep  => ',',
        neg_format     => $FormatAttributes->{'NegFormatNumbers'},
        decimal_digits => GOOGLE_DECIMAL_DIGITS,
        decimal_fill   => GOOGLE_DECIMAL_FILL,
        int_curr_symbol => $CurrencyID,
        curr_pos_format => 'x ',
    );
    return $Formatter->format_price($Price);
}

#========================================================================================
sub getPrice {
    my $self = shift;
    my ($Product) = @_;

    my $Price = $self->productPrice($Product);
    $Price += $Product->getDeposit($self->currencyID, $self->taxModel, 0) // 0;
    $Price += $Product->getEcoParticipation($self->currencyID, $self->taxModel, 0) // 0;
    return $Price;
}

#========================================================================================
sub replaceTabs {
    my $self = shift;
    my ($Value) = @_;

    $Value =~ s/\t/ /g;
    return $Value;
}

#========================================================================================
# function     getExportValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->getExportValues($Object);
#----------------------------------------------------------------------------------------
# description  Calls the implementation of the super class and replaces all tabs by
#               spaces afterwards. Google does not like quoted data in the CSV file, so
#               we remove all tabs (tab is the separator char) and prevent the CSV export
#               driver from quoting any items. see EPG-19210
#----------------------------------------------------------------------------------------
# input        $Object | object to export | object
#----------------------------------------------------------------------------------------
# return       $aValues | list of values | ref.array.string
#========================================================================================
sub getExportValues {
    my $self = shift;
    my ($Object) = @_;

    my $aValues = $self->SUPER::getExportValues($Object);
    return [map {$self->replaceTabs($_)} @$aValues];
}


#========================================================================================

# export column product_type
sub product_type {
    my $self = shift;
    my ($Product) = @_;

    my $ShopID = $Product->getSite()->id;
    foreach my $Category (@{$Product->getWithParent('Categories')}) {
        next if ($Category->instanceOf('CategorySpecialOffer')); # no special offer category
        next if ($Category->parent->id == $ShopID); # not the start page
        return $Category->get('NameOrAlias', $self->languageID);
    }
    return undef;
}

# export column link
sub link {
    my $self = shift;
    my ($Product) = @_;

    my $Shop = $Product->getSite();
    my $URL;
    if ($Shop->isUnity()) {
        my $PortalLanguageID = $self->{'PortalLanguageID'};
        $URL = $self->_productUrlForUnity($Shop, $Product, $PortalLanguageID);
        LogDebug('GoogleBase Link URL: ', { 'Url' => $URL, 'PortalLanguageID' => $PortalLanguageID });
    } else {
        $URL = $self->productUrl($Product);
        LogDebug('GoogleBase Link URL: ', $URL);
    }

    return $URL;
}

# export column image_link
sub image_link {
    my $self = shift;
    my ($Product) = @_;

    my $imageUrl = $self->imageUrl($Product, $self->imageAttribute);
    return undef unless isValid($imageUrl);

    # EPG-19813: mask special chars in image file name
    my $index = rindex($imageUrl, '/');
    if ($index > -1) {
        $imageUrl = substr($imageUrl, 0, $index) . '/' . UrlEncodeValue(substr($imageUrl, $index + 1));
    }
    else {
        $imageUrl = UrlEncodeValue($imageUrl); # can this be possible?
    }
    return $imageUrl;
}

# export column price
sub price {
    my $self = shift;
    my ($Product) = @_;
    my $Price = $self->getPrice($Product);
    my $FormatedPrice = $self->formatPrice($Price);
    return $FormatedPrice;
}

# export column id
sub id {
    my $self = shift;
    my ($Product) = @_;

    my $LocaleID = $self->portalSite->get('LocaleID');

    return $Product->alias . '_' . $LocaleID;
}

# export column title
sub title {
    my $self = shift;
    my ($Product) = @_;

    my $Value = $Product->getWithParent('Name', $self->languageID );

    if (length($Value) > $MaxLengthTitle) {
        Warning('ValueTooLong', {'ProductID'=>$Product->id, 'Product'=>$Product, 'AttributeName' => 'title', 'MaxLength' => $MaxLengthTitle});
        $Value = substr($Value, 0, $MaxLengthTitle);
    }
    $Value =~ s/\"/\'/g;
    return $Value;
}

# export column description
sub description {
    my $self = shift;
    my ($Product) = @_;

    my $DescriptionAttribute = $self->descriptionAttribute;
    if ( defined $DescriptionAttribute ) {
        my $Alias = $DescriptionAttribute->alias;
        my $Value = $Product->getWithParent($Alias, $self->languageID );

        if (defined $Value) {
            $Value = decode_utf8($Value);
            $Value = $self->removeHtmlTags($Value);
            if (length($Value) > MAX_LENGTH_DESCRIPTION) {
                Warning('ValueTooLong', {'ProductID'=>$Product->id, 'Product'=>$Product, 'AttributeName' => 'description', 'MaxLength' => MAX_LENGTH_DESCRIPTION});
                $Value = substr($Value, 0, MAX_LENGTH_DESCRIPTION);
            }
        }
        else {
            Error('ProductNoDescription', {'ProductID' => $Product->id});
        }
        $Value =~ s/\"/\'/g;
        return $Value;
    }
    return;
}

# export column brand
sub brand {
    my $self = shift;
    my ($Product) = @_;

    return $Product->getWithParent('Manufacturer');
}

# export column shipping_weight
sub shipping_weight {
    my $self = shift;
    my ($Product) = @_;

    my $Weight = $Product->getWithParent('Weight');
    return undef unless defined $Weight;

    my $WeightUnitName = undef;
    my $WeightUnit = $Product->getWithParent('WeightUnit');
    if (defined $WeightUnit) {
        # EPG-19587
        # Google does only accept lb, oz, g, kg
        # so we will calculate the weight to a unit Google accepts as follows:
        # mg => g
        # any unit other than lb, oz, g, kg, mg => kg
        my $WeightUnitAlias = $WeightUnit->alias;
        if ($WeightUnitAlias eq 'milligram') {
            $Weight /= 1000; ## no critic qw(ProhibitMagicNumbers)
            $WeightUnit = LoadObjectByPath('/Units/gram');
        }
        elsif (!($WeightUnitAlias ~~ @ACCEPTED_WEIGHT_UNITS)) {
            my $Factor = $WeightUnit->get('Factor');
            if (defined($Factor) && fcmp($Factor, 0) != 0) {
                $Weight = $Weight * $Factor / 1000; ## no critic qw(ProhibitMagicNumbers)
                $WeightUnit = LoadObjectByPath('/Units/gram/kilogram');
            }
        }
        $WeightUnitName = $WeightUnit->get('Abbreviation') // $WeightUnit->alias;
    }

    return defined $WeightUnitName  ?  $Weight . ' ' . $WeightUnitName  :  $Weight;
}

# export column gtin
sub gtin {
    my $self = shift;
    my ($Product) = @_;

    return $Product->getWithParent('UPCEAN');
}

# condition
sub condition {
    my $self = shift;
    my ($Product) = @_;

    return 'new';
}

# shipping
# sub-attributes when not specified are assumed as: "country:region:service:price"
sub shipping {
    my $self = shift;
    my ($Product) = @_;

    my $ShippingPrice = $self->shippingPrice($Product);
    my $FormattedShippingPrice = $self->formatPrice($ShippingPrice);

    my $ProductShippingCountries = $self->_getProductShippingCountries($Product);

    my $GoogleFormattedShipping = '';

    foreach my $Country (@{$ProductShippingCountries}) {
        $GoogleFormattedShipping .= sprintf('%s:::%s,', $Country, $FormattedShippingPrice);
    }

    chop($GoogleFormattedShipping);

    return $GoogleFormattedShipping;
}

# manufacturer product number
sub mpn {
    my $self = shift;
    my ($Product) = @_;

    return $Product->getWithParent('ManufacturerSKU');
}

sub google_product_category {
    my $self = shift;
    my ($Product) = @_;

    my $aPossibleAttributeNames = ['Google_Product_Category', 'google_product_category'];

    my $GoogleProductCategory;
    foreach my $AttributeName (@{$aPossibleAttributeNames}){
        if($Product->get('Class')->existsAttribute($AttributeName)){
            $GoogleProductCategory = $Product->getWithParent($AttributeName, $self->languageID());
            last;
        }
    }
    if ( ! $GoogleProductCategory ) {
        $self->_Warning('ProductNoGoogleCategory', {
            'ProductID'      => $Product->id(),
            'Product'        => $Product,
            'AttributeAlias' => join(' or ', @{$aPossibleAttributeNames}),
        });
        return undef;
    }

    # EPG-19668: only use the first line of attribute value and remove all quotes
    my $AmountOfCharactersBeforeLineBreak = index($GoogleProductCategory, "\n");
    if ($AmountOfCharactersBeforeLineBreak != -1) { # -1 means no match of index
        my $StartPosition = 0;
        $GoogleProductCategory = substr($GoogleProductCategory, $StartPosition, $AmountOfCharactersBeforeLineBreak);
    }
    $GoogleProductCategory =~ s/"//xg; # " # remove quotes
    return $GoogleProductCategory;
}

sub _Warning {
    my $self = shift;
    my ($Code, $hVars) = @_;
    Warning($Code, $hVars);
    return;
}

# availability
sub availability {
    my $self = shift;
    my ($Product) = @_;

    my $StockLevel = $Product->getWithParent('StockLevel');
    my $StockLevelAction = $Product->getSite()->get('ProductLowOnStockAction');

    # The stock level could be empty (if the merchant don't want to fill it) or above zero
    return 'in stock' unless isValid($StockLevel);
    return 'in stock' if $StockLevelAction == STOCK_IS_ORDERABLE;
    return 'in stock' if $StockLevel > 0;

    return 'out of stock';
}

# tax
sub tax {
    my $self = shift;
    my ($Product) = @_;

    return '';

}

# EPG-22192
# The warnings are commented because we dont do wording now
# export column gender
sub gender {
    my $self = shift;
    my ($Product) = @_;

    # US only
    return '' unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCAL_IDS;
    # test if gender set
    unless ($Product->get('Class')->existsAttribute(GENDER_ATTRIBUTE)) {
#        Warning('ProductNoGender', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => GENDER_ATTRIBUTE
#        });
        return undef;
    }

    my $Gender = $Product->getWithParent(GENDER_ATTRIBUTE, $self->languageID);
    unless(isValid($Gender)) {
#        Warning('ProductNoGender', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => GENDER_ATTRIBUTE
#        });
        return undef;
    }
    # We check the type of the attribute to get its value
    if (blessed($Gender) && $Gender->can('get'))
    {
    return $Gender->get('Alias');
    }
    elsif (ref $Gender eq 'ARRAY')
    {
        return '';
    }
    elsif (! ref $Gender)
    {
        return $Gender;
    }

}
# export column age group
sub age_group {
    my $self = shift;
    my ($Product) = @_;


    return '' unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCAL_IDS;
    # test if age group set
    unless ($Product->get('Class')->existsAttribute(AGE_GROUP_ATTRIBUTE)) {
#        Warning('ProductNoAgeGroup', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => AGE_GROUP_ATTRIBUTE
#        });
        return undef;
    }
    my $AgeGroup = $Product->getWithParent(AGE_GROUP_ATTRIBUTE, $self->languageID);
    unless(isValid($AgeGroup)) {
#        Warning('ProductNoAgeGroup', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => AGE_GROUP_ATTRIBUTE
#        });
        return undef;
    }

    # We check the type of the attribute to get its value
    if (blessed($AgeGroup) && $AgeGroup->can('get'))
    {
        return $AgeGroup->get('Alias');
    }
    elsif (ref $AgeGroup eq 'ARRAY')
    {
        return '';
    }
    elsif (! ref $AgeGroup)
    {
    return $AgeGroup;
    }

}

# export column color
sub color {
    my $self = shift;
    my ($Product) = @_;


    return '' unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCAL_IDS;

    # test if color set
    unless ($Product->get('Class')->existsAttribute(COLOR_ATTRIBUTE)) {
#        Warning('ProductNoColor', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => COLOR_ATTRIBUTE
#        });
        return undef;
    }
    my $Color = $Product->getWithParent(COLOR_ATTRIBUTE, $self->languageID);
    unless(isValid($Color)) {
#        Warning('ProductNoColor', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => COLOR_ATTRIBUTE
#        });
        return undef;
    }

    # We check the type of the attribute to get its value
    if (blessed($Color) && $Color->can('get'))
    {
    return $Color->get('Alias');
}
    elsif (ref $Color eq 'ARRAY')
    {
        return '';
    }
    elsif (! ref $Color)
    {
        return $Color;
    }
}
# export column size
sub size {
    my $self = shift;
    my ($Product) = @_;


    return '' unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCAL_IDS;

    # test if size set
    unless ($Product->get('Class')->existsAttribute(SIZE_ATTRIBUTE)) {
#        Warning('ProductNoSize', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => SIZE_ATTRIBUTE
#        });
        return undef;
    }
    my $Size = $Product->getWithParent(SIZE_ATTRIBUTE, $self->languageID);
    unless(isValid($Size)) {
#        Warning('ProductNoSize', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => SIZE_ATTRIBUTE
#        });
        return undef;
    }

    # We check the type of the attribute to get its value
    if (blessed($Size) && $Size->can('get'))
    {
    return $Size->get('Alias');
}
    elsif (ref $Size eq 'ARRAY')
    {
        return '';
    }
    elsif (! ref $Size)
    {
        return $Size;
    }
}
# export column item_group_id, only for variant, with the alias of the super product
sub item_group_id {
    my $self = shift;
    my ($Product) = @_;
    return '' unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCAL_IDS;
    #Check if the product is a variant
    my $SuperProduct = $Product->getWithParent('SuperProduct');
    if (defined($SuperProduct)) {
        # We check the type of the attribute to get its value
        if (blessed($SuperProduct) && $SuperProduct->can('get')) {
            return $SuperProduct->get('Alias');
        }
        elsif (ref $SuperProduct eq 'ARRAY') {
            return '';
        }
        elsif (! ref $SuperProduct) {
            return $SuperProduct;
        }
    } else {
        return '';
    }
}

# export column material
sub material {
    my $self = shift;
    my ($Product) = @_;

    return '' unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCAL_IDS;

    # test if size set
    unless ($Product->get('Class')->existsAttribute(MATERIAL_ATTRIBUTE)) {
        return undef;
    }
    my $Material = $Product->getWithParent(MATERIAL_ATTRIBUTE, $self->languageID);

    return undef unless(isValid($Material));

    # We check the type of the attribute to get its value
    if (blessed($Material) && $Material->can('get'))
    {
        return $Material->get('Alias');
    }
    elsif (ref $Material eq 'ARRAY')
    {
        return '';
    }
    elsif (! ref $Material)
    {
        return $Material;
    }
}
# export column pattern
sub pattern {
    my $self = shift;
    my ($Product) = @_;

    return '' unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCAL_IDS;

    # test if size set
    unless ($Product->get('Class')->existsAttribute(PATTERN_ATTRIBUTE)) {
        return undef;
    }
    my $Pattern = $Product->getWithParent(PATTERN_ATTRIBUTE, $self->languageID);
    return undef unless(isValid($Pattern));

    # We check the type of the attribute to get its value
    if (blessed($Pattern) && $Pattern->can('get'))
    {
        return $Pattern->get('Alias');
    }
    elsif (ref $Pattern eq 'ARRAY')
    {
        return '';
    }
    elsif (! ref $Pattern)
    {
        return $Pattern;
    }
}

#Export identifier exist
sub identifier_exists {
    my $self = shift;
    my ($Product) = @_;

    my $identifierExists = 'FALSE';

    #at least 2 of the following 3 identifiers are required: 'brand', 'gtin', and 'mpn'
    my $aFields =[qw(Manufacturer UPCEAN ManufacturerSKU)];
    my @Information = values(%{$Product->getWithParent($aFields)});
    if ( scalar(grep {isValid($_)} @Information ) >= 2 )
    {
        $identifierExists='TRUE';
    }

    return $identifierExists;
}

sub unit_pricing_measure {
    my $self = shift;
    my ($Product) = @_;

    my $hProductInfo = $Product->getWithParent( [qw( RefContentAmount RefUnit )] );

    my $unit_pricing_measure = '';
    if ($self->_isValidUnitPricingMeasure($hProductInfo)) {
        $unit_pricing_measure = $self->_getUnitPricingMeasure($hProductInfo);
    }
    return $unit_pricing_measure;
}

sub unit_pricing_base_measure{
    my $self = shift;
    my ($Product) = @_;

    my $hProductInfo = $Product->getWithParent( [qw( RefContentAmount RefAmount RefUnit )] );

    my $unit_pricing_base_measure = '';
    if ($self->_isValidUnitPricingBaseMeasure($hProductInfo))
    {
            my $unitAbbreviation = $hProductInfo->{RefUnit}->get( 'Abbreviation' );
            if ($hProductInfo->{RefUnit}->get('Code') eq 'MTK') {
                $unitAbbreviation = 'sqm';
            }
            $unit_pricing_base_measure= sprintf ('%s%s', $hProductInfo->{RefAmount}, $unitAbbreviation);

            $unit_pricing_base_measure = $self->_convertMeasurementFromEpagesToGoogle($unit_pricing_base_measure);
    }

    return $unit_pricing_base_measure;
}

sub is_bundle {
    my $self = shift;
    my ($Product) = @_;

    my $IsBundleProduct = $Product->get('IsBundleProduct');
    my $IsBundle = $IsBundleProduct ? 'TRUE' : 'FALSE';

    return $IsBundle;
}

sub _getProductShippingCountries {
    my $self = shift;
    my ($Product) = @_;

    my $PortalConfig = $self->object();
    my $ProductShippingMethod = $PortalConfig->shippingMethodForProduct($Product);

    if ($ProductShippingMethod->id() == $self->shippingMethod()->id()) {
        return $self->{'PortalShippingMethodCountries'};
    }

    return $self->_getShippingMethodCountries($ProductShippingMethod);
}

sub _getShippingMethodCountries {
    my $self = shift;
    my ($ShippingMethod) = @_;

    if (defined(my $RegionShippingMethod = $ShippingMethod->get('Region'))) {
        my @ShippingCountries;

        foreach my $CountryShippingMethod (@{$RegionShippingMethod->get('Countries')}) {
                push(@ShippingCountries, uc( $CountryShippingMethod->{'Code2'}));
        }
        return \@ShippingCountries;
    }

    return [$self->{'PortalSiteUppercaseCountry'}];
}

sub _isReferenceAmountValid {
    my $self = shift;
    my ($unitAbbreviation, $referenceAmount) = @_;

    return 1 if($referenceAmount ~~ $ALLOWED_AMOUNTS);

    #Additionally, 75cl, 50kg, and 1000kg are accepted.
    return 1 if ($unitAbbreviation eq 'kg' && $referenceAmount ~~ $ALLOWED_EXCEPTIONS_KG_VALUES);
    return 1 if ($unitAbbreviation eq 'ml' && $referenceAmount ~~ $ALLOWED_EXCEPTION_ML_VALUES);

    return 0;
}

sub _convertMeasurementFromEpagesToGoogle {
    my $self = shift;
    my ($ePagesMeasurement) = @_;
    my $googleMeasurement = $ePagesMeasurement;

    #convert 750ml to 75cl
    $googleMeasurement = '75cl' if($ePagesMeasurement eq '750ml');

    return $googleMeasurement;
}

sub _isReferenceAttributesValid {
    my $self    = shift;
    my ($Product) = @_;

    my $hProductInfo = $Product->getWithParent( [qw( RefContentAmount RefAmount RefUnit )] );

    #if the RefUnit and RefContentAmount attributes are null or empty, it is not an error
    if( isValid($hProductInfo->{RefUnit}) &&
        isValid($hProductInfo->{RefContentAmount}))
    {
        $self->_isProductReferenceAmountMisconfigured( $hProductInfo, $Product );
    }
    return;
}

sub _isProductReferenceUnitMisconfigured {
    my $self = shift;
    my ($hProductInfo, $Product) = @_;

    if (isValid($hProductInfo->{RefUnit}->get('Type'))){
        return ($hProductInfo->{RefUnit}->get('Code') ~~ $GOOGLE_VALID_UNIT_CODES ||
            $hProductInfo->{RefUnit}->get('Code') eq 'MLT') ? 1 : 0;
    }
    return 0;
}

sub _isProductReferenceAmountMisconfigured {
    my $self = shift;
    my ($hProductInfo, $Product) = @_;

    #if the RefAmount attribute is null or empty, it is not an error
    if (isValid($hProductInfo->{RefAmount}) && $self->_isProductReferenceUnitMisconfigured( $hProductInfo, $Product ))
    {
        unless( $self->_isReferenceAmountValid( $hProductInfo->{RefUnit}->get( 'Abbreviation' ), $hProductInfo->{RefAmount}))
        {
            Error('ProductReferenceAmountMisconfigured', {
                'ProductID'      => $Product->id,
                'Product'        => $Product,
                'AttributeAlias' => REFERENCE_AMOUNT
            });
        }
    }
}

sub _isValidGoogleWeightMeasurement {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($hProductInfo->{RefUnit}->get('Code') ~~ $GOOGLE_VALID_WEIGHT_UNIT) ? 1 : 0;
}

sub _isValidGoogleVolumeMeasurement {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($hProductInfo->{RefUnit}->get('Code') ~~ $GOOGLE_VALID_VOLUME_UNIT) ? 1 : 0;
}

sub _isValidGoogleAreaMeasurement {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($hProductInfo->{RefUnit}->get('Code') ~~ $GOOGLE_VALID_AREA_UNIT) ? 1 : 0;
}

sub _isValidGoogleLengthMeasurement {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($hProductInfo->{RefUnit}->get('Code') ~~ $GOOGLE_VALID_LENGTH_UNIT) ? 1 : 0;
}

sub _isValidUnitPricingBaseMeasure {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ( isValid($hProductInfo->{RefContentAmount}) &&
        isValid($hProductInfo->{RefUnit}) &&
        isValid($hProductInfo->{RefAmount}) &&
        ($hProductInfo->{RefUnit}->get('Code') ~~ $GOOGLE_VALID_UNIT_CODES ||
            $hProductInfo->{RefUnit}->get('Code') eq 'MLT') &&
        ($hProductInfo->{RefUnit}->get('Type') ne 'area' ||
            $hProductInfo->{RefUnit}->get('Code') eq 'MTK')) ? 1 : 0;
}

sub _isValidUnitPricingMeasure {
    my $self = shift;
    my ($hProductInfo) = @_;
    return (isValid($hProductInfo->{RefContentAmount}) &&
            isValid($hProductInfo->{RefUnit}) &&
            ($hProductInfo->{RefUnit}->get('Type') ~~ $GOOGLE_VALID_UNIT_TYPES) ) ? 1 : 0;
}

sub _getAreaUnitPricing {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($self->_isValidGoogleAreaMeasurement($hProductInfo)) ?
            sprintf ('%ssqm', $hProductInfo->{RefContentAmount}) :
            sprintf ('%ssqm',
                    $hProductInfo->{RefContentAmount} * $hProductInfo->{RefUnit}->get('Factor'));
}

sub _getMassUnitPricing {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($self->_isValidGoogleWeightMeasurement($hProductInfo)) ?
            sprintf ('%s%s', $hProductInfo->{RefContentAmount},
                    $hProductInfo->{RefUnit}->get( 'Abbreviation' )) :
            sprintf ('%sg',
                    $hProductInfo->{RefContentAmount} * $hProductInfo->{RefUnit}->get('Factor'));
}

sub _getVolumeUnitPricing {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($self->_isValidGoogleVolumeMeasurement($hProductInfo)) ?
            sprintf ('%s%s', $hProductInfo->{RefContentAmount},
                    $hProductInfo->{RefUnit}->get( 'Abbreviation' )) :
            sprintf ('%sl',
                    $hProductInfo->{RefContentAmount} * $hProductInfo->{RefUnit}->get('Factor'));
}

sub _getLengthUnitPricing {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($self->_isValidGoogleLengthMeasurement($hProductInfo)) ?
            sprintf ('%s%s', $hProductInfo->{RefContentAmount},
                    $hProductInfo->{RefUnit}->get( 'Abbreviation' )) :
            sprintf ('%sm',
                    $hProductInfo->{RefContentAmount} * $hProductInfo->{RefUnit}->get('Factor'));
}

sub _getUnitPricingMeasure {
    my $self = shift;
    my ($hProductInfo) = @_;
    my $unit_pricing_measure = '';
    if ($hProductInfo->{RefUnit}->get('Type') eq 'area') {
        $unit_pricing_measure = $self->_getAreaUnitPricing($hProductInfo);
    }
    elsif ($hProductInfo->{RefUnit}->get('Type') eq 'mass') {
        $unit_pricing_measure = $self->_getMassUnitPricing($hProductInfo);
    }
    elsif ($hProductInfo->{RefUnit}->get('Type') eq 'volume') {
        $unit_pricing_measure = $self->_getVolumeUnitPricing($hProductInfo);
    }
    elsif ($hProductInfo->{RefUnit}->get('Type') eq 'length') {
        $unit_pricing_measure = $self->_getLengthUnitPricing($hProductInfo);
    }
    return $unit_pricing_measure;
}

sub _productUrlForUnity {
    my $self = shift;
    my ($Shop, $Product, $PortalLanguageID) = @_;

    my $ShopLanguageID = $Shop->get('LanguageID');
    my $UseStoredPrettyUrl = 1;
    my $UrlWithoutExtensions = GetUnityPrettyUrl($Product, $ShopLanguageID, $PortalLanguageID, $UseStoredPrettyUrl);

    # check if there are required extensions to add
    my $PortalConfig = $self->object();

    my $hParams = {
        'PortalConfig' => $PortalConfig,
        'Url'          => $UrlWithoutExtensions
    };
    TriggerHook('PortalExportProductUrl', $hParams);

    return $hParams->{'Url'};
}

1;