#========================================================================================
# package      DE_EPAGES::GoogleBase::t::unit::GoogleBaseProductsExportDriver
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for GoogleBaseProductsExportDriver
#========================================================================================
package DE_EPAGES::GoogleBase::t::unit::GoogleBaseProductsExportDriver;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;
use Test::Mockify;
use Test::Mockify::Matcher qw ( String  HashRef Object Number );
use Test::Mockify::Verify qw ( GetParametersFromMockifyCall WasCalled GetCallCount );
use DE_EPAGES::GoogleBase::API::CSV::GoogleBaseProductsExportDriver;
use strict;

sub test {
    my $self = shift;

    $self->test_google_product_category_Uppercase_export();
    $self->test_google_product_category_Lowercase_export();
    $self->test_google_product_category_export_emptyValue();
    $self->test_google_product_category_export_AttributeDontExists();
    $self->test_google_product_category_removeUnexpectedCharacters();
    $self->test_google_shipping_productShippingEqualsPortalShipping();
    $self->test_google_shipping_productShippingNotEqualsPortalShipping();
    done_testing();
}
#----------------------------------------------------------------------------------------
sub test_google_product_category_Uppercase_export {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Product = $self->_createProduct({
        'Google_Product_Category' => 'TestCategory',
        'ProductTypeAttributes' => [
            'Google_Product_Category',
        ]
    });

    my $Result = $SUT->google_product_category( $Product );

    is( $Result, 'TestCategory', "$SubTestName - prove that the google category is exported." );
    is( WasCalled($SUT, '_Warning'), 0, "$SubTestName - prove that no warning was created." );
}
#----------------------------------------------------------------------------------------
sub test_google_product_category_Lowercase_export {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Product = $self->_createProduct({
        'google_product_category' => 'TestUnityCategory',
        'ProductTypeAttributes' => [
            'google_product_category',
        ]
    });

    my $Result = $SUT->google_product_category( $Product );

    is( $Result, 'TestUnityCategory', "$SubTestName - prove that the google category is exported." );
    is( WasCalled($SUT, '_Warning'), 0, "$SubTestName - prove that no warning was created." );
}
#----------------------------------------------------------------------------------------
sub test_google_product_category_export_emptyValue {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Product = $self->_createProduct({
        'Google_Product_Category' => undef,
        'google_product_category' => undef,
        'ProductTypeAttributes' => [
            'Google_Product_Category',
            'google_product_category',
        ]
    });

    my $Result = $SUT->google_product_category( $Product );

    is($Result, undef, "$SubTestName - prove that the google category is exporting undef.");

    is(GetCallCount($SUT, '_Warning'),1,"$SubTestName - prove that one warning was created.");
    my $aParameters = GetParametersFromMockifyCall($SUT, '_Warning');
    is($aParameters->[0],'ProductNoGoogleCategory',"$SubTestName - prove that the warning name is set.");
    is_deeply(
        $aParameters->[1]->{'AttributeAlias'},
        'Google_Product_Category or google_product_category',
        "$SubTestName - prove that the 'AttributeAlias' is set."
    );
    is_deeply($aParameters->[1]->{'ProductID'}, 666, "$SubTestName - prove that the 'ProductID' is set.");
    isa_ok($aParameters->[1]->{'Product'}, 'DE_EPAGES::Product::API::Object::Product');
}

#----------------------------------------------------------------------------------------
sub test_google_product_category_export_AttributeDontExists {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Product = $self->_createProduct({
        'Google_Product_Category' => undef,
        'google_product_category' => undef,
        'ProductTypeAttributes' => [ ]
    });

    my $Result = $SUT->google_product_category( $Product );

    is($Result, undef, "$SubTestName - prove that the google category is exporting undef.");

    is(GetCallCount($SUT, '_Warning'),1,"$SubTestName - prove that one warning was created.");
    my $aParameters = GetParametersFromMockifyCall($SUT, '_Warning');
    is($aParameters->[0],'ProductNoGoogleCategory',"$SubTestName - prove that the warning name is set.");
    is_deeply(
        $aParameters->[1]->{'AttributeAlias'},
        'Google_Product_Category or google_product_category',
        "$SubTestName - prove that the 'AttributeAlias' is set."
    );
    is_deeply($aParameters->[1]->{'ProductID'}, 666, "$SubTestName - prove that the 'ProductID' is set.");
    isa_ok($aParameters->[1]->{'Product'}, 'DE_EPAGES::Product::API::Object::Product');
}
#----------------------------------------------------------------------------------------
sub test_google_product_category_removeUnexpectedCharacters {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Product = $self->_createProduct({
        'Google_Product_Category' => "\"Test\" Category\n second Line",
        'ProductTypeAttributes' => [
            'Google_Product_Category',
        ]
    });

    my $Result = $SUT->google_product_category( $Product );

    is( $Result, 'Test Category', "$SubTestName - prove that the google category is exported." );
    is( WasCalled($SUT, '_Warning'), 0, "$SubTestName - prove that no warning was created." );
}
#----------------------------------------------------------------------------------------
sub test_google_shipping_productShippingEqualsPortalShipping {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct({
        'Google_Product_Category' => "\"Test\" Category\n second Line",
        'ProductTypeAttributes' => [
            'Google_Product_Category',
        ]
    });

    my $hProductShippingAttributes = {
        'FormattedShippingPrice' => '1 EUR',
        'ShippingMethodId' => 123,
    };

    my $PortalShippingMethod = $self->_createShippingMethod({'ShippingMethodId' => 123});

    my $SUT = $self->_createSUT($hProductShippingAttributes, $PortalShippingMethod);

    $SUT->{'PortalSiteUppercaseCountry'} = 'DE';
    $SUT->{'PortalShippingMethodCountries'} = ['TH'];

    my $ActualResult = $SUT->shipping( $Product );
    my $ExpectedResult = 'TH:::1 EUR';

    is( $ActualResult, $ExpectedResult, "$SubTestName - asserts that default shipping countries are exported" );
    is( WasCalled($SUT, '_getShippingMethodCountries'), 0, "$SubTestName - asserts that _getShippingMethodCountries was not called." );
}

sub test_google_shipping_productShippingNotEqualsPortalShipping {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Product = $self->_createProduct({
        'Google_Product_Category' => "\"Test\" Category\n second Line",
        'ProductTypeAttributes' => [
            'Google_Product_Category',
        ]
    });

    my $aRegionCountries = [
        {'CountryID' => 1, 'Code2' => 'DE'},
        {'CountryID' => 2, 'Code2' => 'AT'},
        {'CountryID' => 3, 'Code2' => 'CH'}
    ];

    my $hProductShippingAttributes = {
        'FormattedShippingPrice' => '2 EUR',
        'ShippingMethodId' => 321,
        'Region' => $aRegionCountries
    };

    my $PortalShippingMethod = $self->_createShippingMethod({'ShippingMethodId' => 123});

    my $SUT = $self->_createSUT($hProductShippingAttributes, $PortalShippingMethod);

    $SUT->{'PortalSiteCountryID'} = 1;
    $SUT->{'PortalSiteUppercaseCountry'} = 'DE';
    $SUT->{'PortalShippingMethodCountries'} = ['TH'];

    my $ActualResult = $SUT->shipping( $Product );
    my $ExpectedResult = 'DE:::2 EUR,AT:::2 EUR,CH:::2 EUR';

    is( $ActualResult, $ExpectedResult, "$SubTestName - asserts that product shipping countries are exported" );
    ok( WasCalled($SUT, '_getShippingMethodCountries'), "$SubTestName - asserts that _getShippingMethodCountries was called." );
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($hAttributes, $PortalShippingMethod) = @_;

    my $aParameterList = [];
    my $Mockify = Test::Mockify->new(
        'DE_EPAGES::GoogleBase::API::CSV::GoogleBaseProductsExportDriver',
        $aParameterList
    );

    $Mockify->spy('_Warning')->when(String(), HashRef());
    $Mockify->spy('_getShippingMethodCountries')->when(Object());
    $Mockify->mock('object')->when()->thenReturn($self->_createPortalConfig($hAttributes));
    $Mockify->mock('shippingPrice')->when(Object())->thenReturn(1);
    $Mockify->mock('formatPrice')->when(Number(1))->thenReturn($hAttributes->{'FormattedShippingPrice'} // 1);
    $Mockify->mock('shippingMethod')->when()->thenReturn($PortalShippingMethod // 1);

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createProduct {
    my $self = shift;
    my ($hValues) = @_;

    my $hAttributes = {
        'ID' => '666',
        'Class' => $self->_createProductType($hValues->{'ProductTypeAttributes'}),
        'Google_Product_Category' => $hValues->{'Google_Product_Category'},
        'google_product_category' => $hValues->{'google_product_category'},
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Product::API::Object::Product',
        $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createProductType {
    my $self = shift;
    my ($aDirectAttributes) = @_;

    my $hAttributes = {
        'Alias' => 'MyShop',
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Product::API::Object::ProductType',
        $hAttributes
    );
    my $Product = $Mockify->getMockObject();
    #This is a hack in order to get the corret result when calling existsAttribute on this class
    foreach my $DirectAttribute (@{$aDirectAttributes}){
        $Product->{'DirectAttributes'}->{$DirectAttribute} = 1;
    }
    return $Product;
}
#----------------------------------------------------------------------------------------
sub _createPortalConfig {
    my $self = shift;
    my ($hAttributes) = @_;

    my $hParameterList = {
    };

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::ProductPortal::API::Object::PortalConfig',
            $hParameterList
    );

    $Mockify->addMockWithReturnValueAndParameterCheck('shippingMethodForProduct', $self->_createShippingMethod($hAttributes), ['object']);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createRegion {
    my $self = shift;
    my ($hAttributes) = @_;

    if (!defined $hAttributes->{'Region'}) {
        return;
    }

    my $hRegionAttributes = {
        'Countries' => $hAttributes->{'Region'}
    };

    my $Mockify =
      DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Region', $hRegionAttributes);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createShippingMethod {
    my $self = shift;
    my ($hAttributes) = @_;

    my $hShippingAttributes = {
        'ID' => $hAttributes->{'ShippingMethodId'},
        'Region' => $self->_createRegion($hAttributes),
    };

    my $Mockify =
      DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shipping::API::Object::ShippingMethod', $hShippingAttributes);

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript();
