#========================================================================================
# package      DE_EPAGES::Order::API::OrderPatch
# state        public
#----------------------------------------------------------------------------------------
# description  Implements JSON-PATCH (RFC 6902) for the order object.
#========================================================================================
package DE_EPAGES::Order::API::OrderPatch;
use base DE_EPAGES::WebService::API::JSONPatch;

use DE_EPAGES::Core::API::PerlTools qw ( GenerateGUID isValid );
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias Transaction );
use DE_EPAGES::Order::API::WebService::OrderService;
use DE_EPAGES::WebService::API::Util qw ( ConvertAddressToPerl );
use DE_EPAGES::Shop::API::AddressValidation;
use DateTime::Format::ISO8601;
use feature 'switch';

use strict;

## no critic (ProhibitUnusedPrivateSubroutines)



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $Order, $LanguageId );
#----------------------------------------------------------------------------------------
# description  constructor
#----------------------------------------------------------------------------------------
# input        $Order | A Product | DE_EPAGES::Order::API::Object::Order
# input        $LanguageId | A LanguageID | integer
# return       $self | self | DE_EPAGES::Order::API::OrderPatch
#========================================================================================
sub new {
    my $class = shift;
    my ($Order, $LanguageId) = @_;
    my $self  = bless {
        'Order' => $Order,
        'LanguageId' => $LanguageId,
    }, $class;
    return $self;
}



sub _add {
    my $self = shift;
    my ($Path, $Value) = @_;

    my $Order = $self->{'Order'};
    my $LanguageId = $self->{'LanguageId'};

    if($Path eq '/orderNumber') {
        $Order->set({'Alias' => $Value}, $LanguageId);
        return;
    }

    if($Path eq '/customerComment') {
        $Order->set({'CustomerComment' => $Value}, $LanguageId);
        return;
    }
    if($Path eq '/internalNote') {
        $Order->set({'Comment' => $Value}, $LanguageId);
        return;
    }

    if($Path eq '/billingAddress') {
        my $hBillingAddressData = ConvertAddressToPerl($Value);
        $self->_setAddress($Order, 'Billing', $hBillingAddressData);
        return;
    }
    if($Path eq '/shippingAddress') {
        my $hShippingAddressData = ConvertAddressToPerl($Value);
        $self->_setAddress($Order, 'Shipping', $hShippingAddressData);
        return;
    }

    if($Path eq '/payment') {
        my $PaymentName = $Value;
        $self->_setPayment($Order, $LanguageId, $PaymentName);
        return
    }

    if($Path eq '/viewedOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'ViewedOn' => $DateTime});
        return;
    }
    if($Path eq '/rejectedOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'CancelledOn' => $DateTime});
        return;
    }
    if($Path eq '/inProcessOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'InProcessOn' => $DateTime});
        return;
    }
    if($Path eq '/pendingOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'PendingOn' => $DateTime});
        return;
    }
    if($Path eq '/readyForDispatchOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'ReadyForShippingOn' => $DateTime});
        return;
    }
    if($Path eq '/partialyDispatchedOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'PartlyDispatchedOn' => $DateTime});
        return;
    }
    if($Path eq '/dispatchedOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'DispatchedOn' => $DateTime});
        return;
    }
    if($Path eq '/deliveredOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'ShippedOn' => $DateTime});
        return;
    }
    if($Path eq '/partialyInvoicedOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'PartlyInvoicedOn' => $DateTime});
        return;
    }
    if($Path eq '/invoicedOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'InvoicedOn' => $DateTime});
        return;
    }
    if($Path eq '/partialyPaidOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'PartlyPaidOn' => $DateTime});
        return;
    }
    if($Path eq '/paidOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'PaidOn' => $DateTime});
        return;
    }
    if($Path eq '/returnedOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'ReturnedOn' => $DateTime});
        return;
    }
    if($Path eq '/closedOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'ClosedOn' => $DateTime});
        return;
    }
    if($Path eq '/archivedOn') {
        my $DateTime = $self->_convertToDatetime($Value);
        $Order->set({'ArchivedOn' => $DateTime});
        return;
    }

    Error('BadRequest', {'Reason' => "Operation add on $Path is not supported!"});
}



sub _remove {
    my $self = shift;
    my ($Path) = @_;

    my $Order = $self->{'Order'};
    my $LanguageId = $self->{'LanguageId'};

    if($Path eq '/customerComment') {
        $Order->set({'CustomerComment' => undef}, $LanguageId);
        return;
    }
    if($Path eq '/internalNote') {
        $Order->set({'Comment' => undef}, $LanguageId);
        return;
    }

    if($Path eq '/viewedOn') {
        $Order->set({'ViewedOn' => undef});
        return;
    }
    if($Path eq '/rejectedOn') {
        $Order->set({'CancelledOn' => undef});
        return;
    }
    if($Path eq '/inProcessOn') {
        $Order->set({'InProcessOn' => undef});
        return;
    }
    if($Path eq '/pendingOn') {
        $Order->set({'PendingOn' => undef});
        return;
    }
    if($Path eq '/readyForDispatchOn') {
        $Order->set({'ReadyForShippingOn' => undef});
        return;
    }
    if($Path eq '/partialyDispatchedOn') {
        $Order->set({'PartlyDispatchedOn' => undef});
        return;
    }
    if($Path eq '/dispatchedOn') {
        $Order->set({'DispatchedOn' => undef});
        return;
    }
    if($Path eq '/deliveredOn') {
        $Order->set({'ShippedOn' => undef});
        return;
    }
    if($Path eq '/partialyInvoicedOn') {
        $Order->set({'PartlyInvoicedOn' => undef});
        return;
    }
    if($Path eq '/invoicedOn') {
        $Order->set({'InvoicedOn' => undef});
        return;
    }
    if($Path eq '/partialyPaidOn') {
        $Order->set({'PartlyPaidOn' => undef});
        return;
    }
    if($Path eq '/paidOn') {
        $Order->set({'PaidOn' => undef});
        return;
    }
    if($Path eq '/returnedOn') {
        $Order->set({'ReturnedOn' => undef});
        return;
    }
    if($Path eq '/closedOn') {
        $Order->set({'ClosedOn' => undef});
        return;
    }
    if($Path eq '/archivedOn') {
        $Order->set({'ArchivedOn' => undef});
        return;
    }
    Error('BadRequest', {'Reason' => "Operation remove on $Path is not supported!"});
}



sub _setAddress {
    my $self = shift;
    my ($Order, $AddressTypePrefix, $hAddressData) = @_;

    if ( isValid($hAddressData) ) {
        $self->_validateAddressData($hAddressData);
        my $AddressType = $AddressTypePrefix . 'Address';
        my $OrderService = $self->_createSoapOrderService();
        $OrderService->setAddress($Order, $hAddressData, $AddressType);
        $self->_setLastUpdate($Order);
    }

    return;
}


sub _setPayment {
    my $self = shift;
    my ($Order, $LanguageId, $PaymentName) = @_;

    my $Container = $Order->child('LineItemContainer');
    $LanguageId //= $Container->get('LanguageID');
    my $LineItemPayment = $Container->get('Payment');
    my $TaxClass = $Container->get('TaxMatrix')->defaultTaxClass();
    my $TaxRate = $Container->taxRate($TaxClass);

    Transaction(
        sub {
            $LineItemPayment->delete();

            my $Payment = $Container->insertLineItem(
                LoadClassByAlias('LineItemPayment'),
                {
                    'Alias'         => 'LineItemPayment_' . GenerateGUID(),
                    'PaymentMethod' => undef,
                    'TaxClass'      => $TaxClass,
                    'TaxRate'       => $TaxRate,
                    'LineItemPrice' => 0,
                }
            );

            $Payment->set({ 'Name' => $PaymentName }, $LanguageId);
        }
    );

    return;
}


sub _validateAddressData {
    my $self = shift;
    my ($hAddressData) = @_;

    my $Order = $self->{'Order'};
    my $User = $Order->get('User');
    my $LocaleID = defined $User ? $User->get('LocaleID') : undef;

    my $AddressValidation = $self->_createAddressValidation($Order, $LocaleID);
    $AddressValidation->validate($hAddressData);
}



sub _convertToDatetime {
    my $self = shift;
    my ($DateTimeString) = @_;

    if(!defined $DateTimeString) {
        return undef;
    }

    return DateTime::Format::ISO8601->parse_datetime( $DateTimeString );
}



sub _createAddressValidation {
    my $self = shift;
    my ($Order, $LocaleID) = @_;
    return DE_EPAGES::Shop::API::AddressValidation->new($Order, $LocaleID);
}



sub _createSoapOrderService {
    return DE_EPAGES::Order::API::WebService::OrderService->new();
}



sub _setLastUpdate {
    my $self = shift;
    my ($Order) = @_;
    my $Profile = $self->_getLastUpdateOrderChangeProfile();
    $Order->setUpdated($Profile);
    return;
}



sub _getLastUpdateOrderChangeProfile {
    my $self = shift;
    return LoadClassByAlias('CustomerOrder')->lastUpdateProfile('OrderChange');
}



1;