#========================================================================================
# package      DE_EPAGES::PayPalExpress::API::Request::URLBuilder
# state        public
#----------------------------------------------------------------------------------------
# description  build the Shop URLs for PayPal
#----------------------------------------------------------------------------------------
# example      my $URLBuilder = DE_EPAGES::PayPalExpress::API::Request::URLBuilder->new($Servlet);
#               my $URL = $URLBuilder->getReturnURL( $BasketOrOrder );
#========================================================================================
package DE_EPAGES::PayPalExpress::API::Request::URLBuilder;
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );
use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( $Servlet );
#----------------------------------------------------------------------------------------
# description  constuctor
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | DE_EPAGES::Presentation::Servlet::PageType
# return       $self | self | DE_EPAGES::PayPalExpress::API::Request::URLBuilder
#========================================================================================
sub new {
    my $class = shift;
    my ($Servlet)  = @_;
    my $self  = bless {
        'Servlet' => $Servlet,
    }, $class;
    return $self;
}

#========================================================================================
# function     getReturnURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       getReturnURL( $BasketOrOrder );
#----------------------------------------------------------------------------------------
# description  give the Return URL for paypal
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | basket or order | object
# return       $URL | return URL |string
#========================================================================================
sub getReturnURL {
    my $self = shift;
    my ( $BasketOrOrder ) = @_;

    my $hOptions = $self->_getBuildShopURLOptions();
    my $URL = $self->_BuildShopUrl(
         $BasketOrOrder,
         {'ChangeAction' => ['BlockMonitor', 'PayPalExpressReturn']},
         $hOptions,
         $self->{'Servlet'}
    );

    return $URL;
}

#========================================================================================
# function     getCancelURLCustomerOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       getCancelURLCustomerOrder( $CustomerOrder );
#----------------------------------------------------------------------------------------
# description  give the cancel URL for paypal for Customer order
#----------------------------------------------------------------------------------------
# input        $CustomerOrder | Customer order | DE_EPAGES::Order::API::Object::CustomerOrder
# return       $URL | cancel URL |string
#========================================================================================
sub getCancelURLCustomerOrder {
    my $self = shift;
    my ( $CustomerOrder ) = @_;

    my $hOptions = $self->_getBuildShopURLOptions();
    my $URL = $self->_BuildShopUrl(
         $CustomerOrder,
         {'ChangeAction' => ['BlockMonitor', 'PayPalExpressCancelOnOrder'] },
         $hOptions,
         $self->{'Servlet'}
    );

    return $URL;
}

#========================================================================================
# function     getCancelURLBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       getCancelURLBasket( $Basket );
#----------------------------------------------------------------------------------------
# description  give the cancel URL for paypal for basket
#----------------------------------------------------------------------------------------
# input        $Basket | basket | DE_EPAGES::Order::API::Object::Basket
# return       $URL | cancel URL |string
#========================================================================================
sub getCancelURLBasket {
    my $self = shift;
    my ( $Basket ) = @_;

    my $hOptions = $self->_getBuildShopURLOptions();
    my $URL = $self->_BuildShopUrl(
         $Basket,
         {'ChangeAction' => [ 'BlockMonitor', 'PayPalExpressCancel' ] },
         $hOptions,
         $self->{'Servlet'}
    );

    return $URL;
}

#========================================================================================
# function     getGiroPaySuccessURLCustomerOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       getGiroPaySuccessURLCustomerOrder( $CustomerOrder );
#----------------------------------------------------------------------------------------
# description  give the GiroPaySuccessURL for paypal for Customer order
#----------------------------------------------------------------------------------------
# input        $CustomerOrder | Customer order | DE_EPAGES::Order::API::Object::CustomerOrder
# return       $URL | GiroPaySuccess URL |string
#========================================================================================
sub getGiroPaySuccessURLCustomerOrder {
    my $self = shift;
    my ( $CustomerOrder ) = @_;

    my $hOptions = $self->_getBuildShopURLOptions();
    my $URL = $self->_BuildShopUrl(
         $CustomerOrder,
         {'ChangeAction' => 'BlockMonitor'},
         $hOptions,
         $self->{'Servlet'}
    );

    return $URL;
}

#========================================================================================
# function     getGiroPaySuccessURLBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       getGiroPaySuccessURLBasket( $Basket, $GUID );
#----------------------------------------------------------------------------------------
# description  give the GiroPaySuccess URL for paypal for basket
#----------------------------------------------------------------------------------------
# input        $Basket | basket | DE_EPAGES::Order::API::Object::Basket
# input        $GUID | GUID | String
# return       $URL | GiroPaySuccess URL |string
#========================================================================================
sub getGiroPaySuccessURLBasket {
    my $self = shift;
    my ( $Basket, $GUID ) = @_;

    my $hOptions = $self->_getBuildShopURLOptions();
    my $URL = $self->_BuildShopUrl(
         $Basket,
         {'ChangeAction' => ['BlockMonitor', 'PayPalExpressGiroPaySuccess'], 'ppeguid' => $GUID},
         $hOptions,
         $self->{'Servlet'}
    );

    return $URL;
}

#========================================================================================
# function     getGiroPayCancelURLBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       getGiroPayCancelURLBasket( $Shop, $GUID );
#----------------------------------------------------------------------------------------
# description  give the GiroPayCancel URL for paypal for basket
#----------------------------------------------------------------------------------------
# input        $Shop | Shop object | DE_EPAGES::Shop::API::Object::Shop
# input        $GUID | GUID | String
# return       $URL | GiroPayCancel URL |string
#========================================================================================
sub getGiroPayCancelURL {
    my $self = shift;
    my ( $Shop, $GUID ) = @_;

    my $hOptions = $self->_getBuildShopURLOptions();
    my $URL = $self->_BuildShopUrl(
         $Shop,
         {'ChangeAction' => ['BlockMonitor', 'PayPalExpressGiroPayCancel'], 'ppeguid' => $GUID},
         $hOptions,
         $self->{'Servlet'}
    );

    return $URL;
}

#========================================================================================
# function     getCallbackURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       getCallbackURL( $BasketOrOrder, $GUID );
#----------------------------------------------------------------------------------------
# description  give the GiroPayCancel URL for paypal for basket
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | basket or order | object
# input        $GUID | GUID | String
# return       $URL | GiroPayCancel URL |string
#========================================================================================
sub getCallbackURL {
    my $self = shift;
    my ( $BasketOrOrder, $GUID ) = @_;

    my $hOptions = $self->_getBuildShopURLOptions();
    my $URL = $self->_BuildShopUrl(
         $BasketOrOrder,
         { 'ChangeAction' => ['BlockMonitor', 'PayPalExpressCallback'], 'BasketGUID' => $GUID },
         $hOptions,
         $self->{'Servlet'}
    );

    return $URL;
}

#========================================================================================
# function     _getBuildShopURLOptions
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getBuildShopURLOptions( );
#----------------------------------------------------------------------------------------
# description  summarize the hOptions for BuildShopURL
#----------------------------------------------------------------------------------------
# return       $hOptions | BuildShopURL options | refhash
#========================================================================================
sub _getBuildShopURLOptions {
    my $self = shift;

    my $hOptions = {
            'UseSSL' => 1,
            'AddToken' => 1,
            'Type' => 'sf/mobile'
    };

    return $hOptions;
}

#========================================================================================
# function     _BuildShopUrl
# state        private
#----------------------------------------------------------------------------------------
# syntax       _BuildShopUrl( $Object, $hParams, $hOptions, $Servlet );
#----------------------------------------------------------------------------------------
# description  wrapper to hide the core binding
#----------------------------------------------------------------------------------------
# input        $Object | any object that belongs to a shop | object
# input        $hParams | url get parameters; values may be strings or arrays of strings | ref.hash
# input        $hOptions | see Original function | ref.hash
# input        $Servlet | the current servlet | object
# return       $url | the url | string
#========================================================================================
sub _BuildShopUrl {
    my $self = shift;
    my ($Object, $hParams, $hOptions, $Servlet) = @_;
    return BuildShopUrl($Object, $hParams, $hOptions, $Servlet);
}

1;