#========================================================================================
# package      DE_EPAGES::Shipping::API::Resource::ShippingMethod
# state        public
# base         DE_EPAGES::WebService::API::Resource::BaseObjectResource
# description  implements the REST resource for a single shipping method.
#========================================================================================
package DE_EPAGES::Shipping::API::Resource::ShippingMethod;
use base DE_EPAGES::WebService::API::Resource::BaseObjectResource;

use strict;
use feature 'switch';
use DE_EPAGES::Core::API::Error qw (Error ExistsError GetError);
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::Url qw (ParseQueryString);
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath);
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::Shop::API::Resource::ShopRequestContext;

sub service {
    return LoadObjectByPath('/RestServices/ShippingMethod');
}

sub class {
    return 'ShippingMethod';
}

sub producedContentTypes {
    shift;
    my ($RequestMethod) = @_;

    return {
        GET => [
            ['get', 'application/vnd.epages.v1+json'],
            ['get', 'application/json'],
        ],
        DELETE => [
            ['delete', undef],
        ],
    }->{$RequestMethod};
}

sub match {
    my $self = shift;
    my ($Request) = @_;

    return unless $Request->uri->path =~ m|^/api/shipping-methods/([^/]+)/?$|;
    return $1;
}

sub params {
    my $self = shift;
    my ($ActionName, $Object, $Request) = @_;

    given( $ActionName ) {
        when( 'get' ) {
            my $hQuery = ParseQueryString( $Request->uri->query, 'utf-8' );
            my $LocaleID = $hQuery->{'locale'}->[0];
            return [ $Object, $LocaleID ];
        }
        default {
            return [ $Object ];
        }
    }
}

sub get {
    my $self = shift;
    my ($ShippingMethod, $LocaleID) = @_;

    Error('ResourceNotFound', { 'Reason' => 'NotActivated'}) unless $ShippingMethod->get('IsActivated');

    # validate query parameters
    my $Context = DE_EPAGES::Shop::API::Resource::ShopRequestContext->new($ShippingMethod->getSite);
    $LocaleID = $Context->validLocale($LocaleID);

    my $Locale= DE_EPAGES::Object::API::Object::Locale->load($LocaleID);
    my $LanguageID = $Locale->get('LanguageID');

    my @Attributes = qw(GUID NameOrAlias Note DisplayLogo Image WebPath );
    my $hShippingMethod = $ShippingMethod->get(\@Attributes, $LanguageID);

    my $LogoURL = $hShippingMethod->{'DisplayLogo'} && $hShippingMethod->{'Image'}
        ? $Context->imageBaseUrl . $hShippingMethod->{'WebPath'} .'/'. $hShippingMethod->{'Image'}
        : undef;
    my $BasePrice = $self->_getBasePrice($ShippingMethod, $LocaleID);

    return {
        shippingMethodId => $hShippingMethod->{'GUID'},
        name => $hShippingMethod->{'NameOrAlias'},
        description => $hShippingMethod->{'Note'},
        logo => $LogoURL,
        price => $BasePrice
    };
}

sub delete {
    my $self = shift;
    my ($ShippingMethod) = @_;

    return unless defined $ShippingMethod;
    $ShippingMethod->delete();

    return;
}

sub _getBasePrice {
    my $self = shift;
    my ($ShippingMethod, $LocaleID) = @_;

    my $BasePrice;
    eval{
        my $TaxModel = $ShippingMethod->getSite()->get('TaxModel');
        my $CurrencyID = $ShippingMethod->getSite()->get('CurrencyID');
        my $Formatter = DE_EPAGES::Shop::API::Resource::Formatter->new($ShippingMethod->getSite(), $LocaleID);

        my $ahLevels = $ShippingMethod->getLevels($CurrencyID, $TaxModel);
        return unless scalar @$ahLevels;

        # The API returns only one price, use first level
        my $hLevel = @$ahLevels->[0];
        my $BaseValue = $hLevel->{'BaseValue'} // undef;
        if(defined($BaseValue)) {
            $BasePrice =  $Formatter->price( $BaseValue, $CurrencyID, $TaxModel );
        };
    };
    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
    }

    return $BasePrice;
}

1;
