#========================================================================================
# package      DE_EPAGES::ShopMigration::API::LegalPagesCommon
# state        public
#----------------------------------------------------------------------------------------
# description common functions for exporting/importing legalpages
#========================================================================================
package DE_EPAGES::ShopMigration::API::LegalPagesCommon;
use base qw(Exporter);

use strict;
our @EXPORT_OK = qw(
    GetLegalPages
    SetLegalPages
    GetLegalPageData
    SetLegalPageData
);

use feature 'switch';
use File::Spec::Functions;
use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Core::API::Error qw(Error);
use DE_EPAGES::Object::API::Factory qw(LoadObject);
use DE_EPAGES::Object::API::Template qw(ProcessFileTemplate);
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode);
use DE_EPAGES::ShopMigration::API::Cartridge;
use DE_EPAGES::ShopMigration::API::Constants qw(
    @LEGELPAGES_ALIASES
    LEGALPAGE_ATTRIBUTES_FOR_BASE_EXPORT
    LEGALPAGE_ATTRIBUTES_LOCALIZEDATTRIBUTES_FOR_BASE_EXPORT
    LEGALPAGE_ATTRIBUTES_SCRIBEATTRIBUTES_FOR_BASE_EXPORT
    LEGALPAGE_ATTRIBUTES_FOR_PREP_EXPORT
    LEGALPAGE_ATTRIBUTES_LOCALIZEDATTRIBUTES_FOR_PREP_EXPORT
);
use DE_EPAGES::ShopMigration::API::HTMLSanitizer qw(SanitizeHTMLForScribe);
use DE_EPAGES::ShopMigration::API::ShopCommonData qw(GetActiveShopLanguages);
use DE_EPAGES::ShopMigration::API::ShortURL qw(InsertRedirect GetAllUILocalesForShop);

#========================================================================================
# function     GetLegalPages
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $ahLegalPages = GetLegalPages($Shop);
#----------------------------------------------------------------------------------------
# description  gets all relevant data of legal pages of a shop
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
#----------------------------------------------------------------------------------------
# return       $ahLegalPages | see GetShopLegalPageData() | reference to array of hashes
#========================================================================================
sub GetLegalPages {
    my ($Shop) = @_;

    my @Items;
    my $IsPrepShop = $Shop->get('IsMigrationPrepShop');
    my $Folder = $IsPrepShop ? $Shop->child('LegalPagesUnity') : $Shop->child('Categories');
    foreach my $Alias (@LEGELPAGES_ALIASES) {
        my $PageID = $Folder->findChild($Alias);
        next unless defined $PageID;
        my $hInfo = GetShopLegalPageData($Shop, LoadObject($PageID));
        push @Items, $hInfo if (keys %$hInfo);
    }
    return \@Items;
}

#========================================================================================
# function     GetLegalPageData
# state        public
#----------------------------------------------------------------------------------------
# syntax       my $hLegalPage = GetLegalPageData($Shop, $LegalPage);
#----------------------------------------------------------------------------------------
# description  Constructs a hash with the exportable fields for a single LegalPage
#----------------------------------------------------------------------------------------
# input        $Shop | shop | object
# input        $LegalPage | LegalPage | object
#----------------------------------------------------------------------------------------
# return       $hLegalPage | hash with following fields
#               <ul>
#                   <li>Attributes
#                       <ul>
#                           <li>Alias                    - string</li>
#                           <li>IsVisible                - boolean (if $Shop is PrepShop only)</li>
#                       </ul>
#                   </li>
#                   <li>LocalizedAttributes
#                       <ul>
#                           <li>LanguageCode2            - en|de|..</li>
#                               <ul>
#                                   <li>Text             - string</li>
#                               </ul>
#                           </li>
#                       </ul>
#                   </li>
#               </ul>| reference to hash
#========================================================================================
sub GetLegalPageData {
    my ($Shop, $LegalPage) = @_;

    my $IsPrepShop = $Shop->get('IsMigrationPrepShop');
    my ($Attributes, $LocalizedAttributes, $ScribeAttributes) =
        _selectAttributesArrayRefsForLegalPageData($IsPrepShop);
    my $hValues = { Attributes => $LegalPage->get($Attributes), LocalizedAttributes => {}, };

    my $ActiveLanguages = GetActiveShopLanguages($Shop);
    foreach my $hLanguage (@$ActiveLanguages) {
        my $AttrInfo = $LegalPage->get($LocalizedAttributes, $hLanguage->{'LanguageID'});
        #base to prep: add description to text, reset description
        unless ($IsPrepShop) {
            $AttrInfo->{'Text'} = _getLegalPageText($LegalPage, $hLanguage);
            delete $AttrInfo->{'Description'};
            delete $AttrInfo->{'NameOrAlias'};
        }
        foreach my $FieldName (@$ScribeAttributes) {
            $AttrInfo->{$FieldName} = SanitizeHTMLForScribe($AttrInfo->{$FieldName});
        }
        $hValues->{LocalizedAttributes}{$hLanguage->{'Code2'}} = $AttrInfo;
    }
    return $hValues;
}

sub _getLegalPageText {
    my ($LegalPage, $hLanguage) = @_;

    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $hLanguage->{'LanguageID'} if defined $hLanguage->{'LanguageID'};
    my $hVars = $LegalPage->tleHash;

    my $Cartridge = DE_EPAGES::ShopMigration::API::Cartridge->new;
    my $Template = catfile($ENV{EPAGES_CARTRIDGES}, $Cartridge->cartridgeDirectory, 'Templates', 'LegalPage.txt');
    my $Text = ProcessFileTemplate($Template, $hLanguage->{'Code2'}, $hVars);
    return br2p($Text);
}

sub br2p {
    my ($Text) = @_;
    $Text =~ s,<br\s*[/]?>,</p><p>,g;
    return $Text;
}

#========================================================================================
# function     SetLegalPages
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetLegalPages($Shop, $ahLegalPages);
#----------------------------------------------------------------------------------------
# description  sets attributes on shopLegalPages as specified in the $ahLegalPages
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $ahLegalPages  | see GetLegalPageData() | reference to array of hashes
#========================================================================================
sub SetLegalPages {
    my ($Shop, $ahLegalPages) = @_;

    foreach my $hLegalPage (@$ahLegalPages) {
        SetLegalPageData($Shop, $hLegalPage);
    }
}

#========================================================================================
# function     SetLegalPageData
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetLegalPageData($Shop, $hLegalPage);
#----------------------------------------------------------------------------------------
# description  sets attributes on a LegalPage as specified in the hash $hLegalPage
#----------------------------------------------------------------------------------------
# input        $Shop         | baseshop or prepshop | object
# input        $hLegalPage    | see GetLegalPageData() | reference to hash
#========================================================================================
sub SetLegalPageData {
    my ($Shop, $hLegalPage) = @_;

    my $Alias = $hLegalPage->{Attributes}{Alias};
    my $LegalPage = $Shop->get($Alias);
    Error('LEGALPAGE_NOTFOUND') unless defined $LegalPage;

    my $IsPrepShop = $Shop->get('IsMigrationPrepShop') ? 0 : 1;
    # Just for readability
    my $IsFinalMigration = $Shop->get('IsMigrationPrepShop') ? 0 : 1;
    my ($Attributes, $LocalizedAttributes) = _selectAttributesArrayRefsForLegalPageData($IsPrepShop);
    my $OldURI = {};
    foreach my $Type (keys %$hLegalPage) {
        given ($Type) {
            when ('Attributes') {
                foreach my $Key (keys %{$hLegalPage->{Attributes}}) {
                    die "Invalid Element for Attributes: $Key" unless ElementOf($Key, @$Attributes);
                }
                $LegalPage->set($hLegalPage->{Attributes});
            }
            when ('LocalizedAttributes') {
                foreach my $LanguageCode2 (keys %{$hLegalPage->{LocalizedAttributes}}) {
                    foreach my $Key (keys %{$hLegalPage->{LocalizedAttributes}{$LanguageCode2}}) {
                        if ($Key eq 'URI' && $IsFinalMigration) {
                            my $Value = delete $hLegalPage->{LocalizedAttributes}{$LanguageCode2}{$Key};
                            $OldURI->{$LanguageCode2} = $Value;
                        }
                        die "Invalid Element for Attributes: $Key" unless ElementOf($Key, @$LocalizedAttributes);
                    }
                    my $LanguageID = GetPKeyLanguageByCode($LanguageCode2);
                    # Silently ignore fields for non-existing languages
                    if ($Shop->existsLanguage($LanguageID)) {
                        $LegalPage->set($hLegalPage->{LocalizedAttributes}{$LanguageCode2}, $LanguageID);
                    }
                }
            } default {
                die "Invalid Type: $Type";
            }
        }
    }

    # add redirect entries for each URI to epages-ui on final migration
    if ($IsFinalMigration) {
        my $aLocales = GetAllUILocalesForShop($Shop);
        foreach my $LocaleID ( @$aLocales ) {
            my ($LanguageCode2) = ($LocaleID =~ /^(\w{2})/);
            my $FromSlug = $OldURI->{$LanguageCode2};
            if (defined $FromSlug) {
                my $ClassAlias = $LegalPage->class->alias;
                my $ToSlug =  '/' . $LanguageCode2;
                given ($ClassAlias) {
                    when ('Imprint') {$ToSlug .= '/l/contact';}
                    when ('TermsAndConditions') {$ToSlug .= '/l/tac';}
                    when ('CustomerInformation') {$ToSlug .= '/l/withdrawal';}
                    when ('PrivacyPolicy') {$ToSlug .= '/l/privacy';}
                    when ('ShippingAndPaymentInformation') {$ToSlug .= '/l/shipping';}
                    when ('CookiePolicy') {$ToSlug .= '/l/cookiepolicy';}
                }
                if (defined $ToSlug && ($FromSlug ne $ToSlug)) {
                    InsertRedirect($Shop, $FromSlug, $ToSlug);
                }
            }
        }
    }
}


# Helper function that copies the proper (constant) ArraysRefs depending to use on Get/Set
# 1 -> use Constants named *PREP* (PrepExport and BaseImport)
# 0 -> use Constants named *BASE* (BaseExport and PrepImport)
sub _selectAttributesArrayRefsForLegalPageData {
    my ($IsPrepShop) = @_;

    my ($Attributes, $LocalizedAttributes, $ScribeAttributes);
    if ($IsPrepShop) {
        $Attributes = [ @{LEGALPAGE_ATTRIBUTES_FOR_PREP_EXPORT()} ];
        $LocalizedAttributes = [ @{LEGALPAGE_ATTRIBUTES_LOCALIZEDATTRIBUTES_FOR_PREP_EXPORT()} ];
        $ScribeAttributes = [];
    } else {
        $Attributes = [ @{LEGALPAGE_ATTRIBUTES_FOR_BASE_EXPORT()} ];
        $LocalizedAttributes = [ @{LEGALPAGE_ATTRIBUTES_LOCALIZEDATTRIBUTES_FOR_BASE_EXPORT()} ];
        $ScribeAttributes = [ @{LEGALPAGE_ATTRIBUTES_SCRIBEATTRIBUTES_FOR_BASE_EXPORT()} ];
    }
    return ($Attributes, $LocalizedAttributes, $ScribeAttributes);
}

1;
