use strict;
use Getopt::Long;
use Time::HiRes qw(time);
use DE_EPAGES::Cartridge::API::Table::Cartridge qw(ExistsCartridgeByPackage);
use DE_EPAGES::Core::API::Error qw(ExistsError GetError);
use DE_EPAGES::Core::API::Log qw(LogError);
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Database::API::Connection qw(RunOnStore);
use DE_EPAGES::Object::API::Language qw(GetCodeByLanguageID);
use DE_EPAGES::Object::API::Factory qw(LoadObject LoadRootObject ExistsObject);
use DE_EPAGES::Object::API::Table::Locale qw(GetLanguageByLocaleID);
use DE_EPAGES::Product::DAL::Product qw(DAL_GetAllProductIDsAliasOrderedByShopID); ##no critic (RequireCorrectImports)
use DE_EPAGES::ShopMigration::API::ShortURL qw(InsertRedirect GetAllUILocalesForShop ExistsRedirect IsAllowedShortURL AddDeletedShortUrlRedirections);
use DE_EPAGES::ShopMigration::API::Constants qw(SHOPMIGRATION_FINAL_DONE);
use DE_EPAGES::WebInterface::API::MessageCenter qw(SynchronizeCache);

my ($Output, $TestMode);

RunScript('Sub' => \&Main);

sub Main {
    local $| = 1;
    my %Options = ();

    my @GetOpt = qw(help passwd=s storename=s shop=s output! test!);
    usage("Unkown parameter specified.\n") unless GetOptions(\%Options, @GetOpt);
    usage() if $Options{help};
    $Output = defined $Options{output} ? $Options{output} : 1;
    $TestMode = defined $Options{test} ? $Options{test} : 1;
    for (qw(storename shop)) {
        usage(sprintf('Missing parameter -%s', $_)) unless $Options{$_};
    }

    my $time1 = time;
    RunOnStore(
        Store      => $Options{storename},
        DBPassword => $Options{passwd},
        Sub        => sub {
            my $ShopAlias = $Options{shop};
            return unless (ExistsCartridgeByPackage('DE_EPAGES::ShopMigration'));
            my $ShopFolder = LoadRootObject()->child('Shops');
            if (!$ShopFolder->existsChild($ShopAlias)) {
                printf("Shop is not existing: %s\n", $ShopAlias);
                exit 2;
            }
            my $Shop = $ShopFolder->child($ShopAlias);
            if (!$Shop->get('IsMigrationShop')) {
                printf("Shop is not a MigrationShop: %s\n", $ShopAlias);
                exit 2;
            }
            my $Status = $Shop->get('MigrationStatus');
            if ($Status != SHOPMIGRATION_FINAL_DONE) {
                printf("Shop is not in proper state (must be %s | is : %s)\n", SHOPMIGRATION_FINAL_DONE, $Status);
                exit 2;
            }
            AddShortURLRedirects($Shop);
        }
    );
    eval {SynchronizeCache();};
    LogError(GetError()) if ExistsError();
    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1 if $Output;
}

sub AddShortURLRedirects {
    my ($Shop) = @_;

    my $aLocales = GetAllUILocalesForShop($Shop);
    printf("%sWorking on Shop %s with locales %s\n",
        $TestMode ? '[TEST] ' : '', $Shop->alias, join(', ', @$aLocales)) if ($Output || $TestMode);
    my $TotalRedirects = 0;

    $TotalRedirects += AddShortURLRedirectsShop($Shop, $aLocales);
    $TotalRedirects += AddShortURLRedirectsProducts($Shop, $aLocales);
    $TotalRedirects += AddShortURLRedirectsCategories($Shop, $aLocales);

    printf("%sInserted %s new redirects for shop %s\n",
        $TestMode ? '[TEST] ' : '', $TotalRedirects, $Shop->alias) if ($Output || $TestMode);
}

sub GetProductsIterator {
    my ($Shop) = @_;

    my $Iterator = DE_EPAGES::Core::API::Iterator->wrap(
        Source => DAL_GetAllProductIDsAliasOrderedByShopID($Shop->id),
        Handle => sub {
            return $_[1]->[0];
        }
    );
    return $Iterator;
}

sub AddShortURLRedirectsShop {
    my ($Shop, $aLocales) = @_;

    my $NewRedirects = 0;
    foreach my $LocaleID (@$aLocales) {
        my $LanguageID = GetLanguageByLocaleID($LocaleID);
        my $FromSlug = $LocaleID;
        if (defined $FromSlug) {
            my $ToSlug = '/' . GetCodeByLanguageID($LanguageID);
            $NewRedirects += _addValidRedirect($Shop, undef, $FromSlug, $ToSlug);
        }
    }
    return $NewRedirects;
}

sub AddShortURLRedirectsProducts {
    my ($Shop, $aLocales) = @_;

    my $NewRedirects = 0;
    my $Iterator = GetProductsIterator($Shop);
    while (my $ProductID = <$Iterator>) {
        next unless (ExistsObject($ProductID));
        my $Product = LoadObject($ProductID);
        foreach my $LocaleID (@$aLocales) {
            my $LanguageID = GetLanguageByLocaleID($LocaleID);
            my $FromSlug = $Product->get('URI', $LanguageID);
            if (defined $FromSlug) {
                my $ToSlug = '/' . GetCodeByLanguageID($LanguageID) . '/p/' . $Product->guid;
                $NewRedirects += _addValidRedirect($Shop, $Product, $FromSlug, $ToSlug);
            }
        }
    }
    return $NewRedirects;
}

sub GetCategories {
    my ($Shop) = @_;

    my @Items;
    my $SubCategories = $Shop->get('Categories')->get('RecursiveSubCategories');
    foreach my $Category (@$SubCategories) {
        my $hInfo = GetCategoryData($Shop, $Category);
        push @Items, $hInfo if (keys %$hInfo);
    }
    return \@Items;
}

sub AddShortURLRedirectsCategories {
    my ($Shop, $aLocales) = @_;

    my $NewRedirects = 0;
    my $SubCategories = $Shop->get('Categories')->get('RecursiveSubCategories');
    unshift @$SubCategories, $Shop->get('Categories');
    foreach my $Category (@$SubCategories) {
        foreach my $LocaleID (@$aLocales) {
            my $LanguageID = GetLanguageByLocaleID($LocaleID);
            my $FromSlug = $Category->get('URI', $LanguageID);
            if (defined $FromSlug) {
                my $ToSlug = '/' . GetCodeByLanguageID($LanguageID) . '/c/' . $Category->guid;
                $NewRedirects += _addValidRedirect($Shop, $Category, $FromSlug, $ToSlug);
            }
        }
    }
    # add old redirect after adding all live ones (collission!)
    AddDeletedShortUrlRedirections($Shop);
    return $NewRedirects;
}

sub _addValidRedirect {
    my ($Shop, $Object, $FromSlug, $ToSlug) = @_;

    my $RedirectAdded = 0;
    my $Exists = ExistsRedirect($Shop, $FromSlug);
    my $Allowed = IsAllowedShortURL($FromSlug);
    unless ($Exists || $TestMode) {
        $RedirectAdded++;
        InsertRedirect($Shop, $FromSlug, $ToSlug) if $Allowed;
    }
    printf "%sFound %s: '%s' with URI: '%s' - %s (%s)\n",
        $TestMode ? '[TEST] ' : '',
        $ToSlug =~ /^[\/]\S{2}[\/]c[\/]/ ? 'Category' : ($ToSlug =~ /^[\/]\S{2}[\/]p[\/]/ ? 'Product' : 'Other'),
        defined $Object ? $Shop->relativePath($Object) : 'ShopData', $FromSlug, $ToSlug,
        $Allowed ? ($Exists ? 'already exists' : 'inserted') : 'not allowed' if ($Output || $TestMode);
    return $RedirectAdded;
}

sub usage {
    my ($ErrorNote) = @_;

    print STDERR "\n$ErrorNote\n\n" if $ErrorNote;
    print <<END_USAGE;
Adds old shorturl redirects from products and categories to the rewrite table (epages-ui)

Usage:
    perl $0 [options] flags objects

options:
    -passwd     database user password                        ()
    -shop       shop alias                                    required
    -storename  name of store                                 required
flags:
    -help       show the command line options
    -(no)test   enable/disable testmode                       (1)
    -(no)output enable/disable verbose mode                   (1)

Example:
    perl $0 -storename Store -shop DemoShop
END_USAGE
    exit 2;
}

1;
