#========================================================================================
# package      DE_EPAGES::ShopMigration::t::CategoriesExportImport
# base         DE_EPAGES::Test::API::UnitTest::Test
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::ShopMigration::API::JSONLines::CategoriesExportImport
#========================================================================================
package DE_EPAGES::ShopMigration::t::CategoriesExportImport;
use base DE_EPAGES::Test::API::StoreUnitTest;

use strict;
use utf8;
use warnings;
use File::Spec::Functions;
use DE_EPAGES::Core::API::File qw(GetFileContent GetFileContentLines ExistsFile DeleteDirectory DeleteFile Text2Unix GetFileNames WriteFile);
use DE_EPAGES::Core::API::PerlTools qw(ElementOf);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadClassByAlias ExistsObjectByPath LoadObjectByPath);
use DE_EPAGES::Object::API::Language qw(GetPKeyLanguageByCode);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw(ImportObjects);
use DE_EPAGES::ShopMigration::API::Constants qw(CATEGORIES_ATTRIBUTES_FOR_RESET CATEGORIES_LOCALIZEDATTRIBUTES_FOR_RESET);
use DE_EPAGES::ShopMigration::API::JSON qw(DecodeJSON DecodeJSONArray);
use DE_EPAGES::ShopMigration::API::JSONLines::CategoriesExportImport qw(ExportCategoriesJSON ImportCategoriesJSON);
use DE_EPAGES::ShopMigration::API::MigrationShop qw(CreatePrepShop ExistsPrepShop SetShopFeatureValues TransferEpagesUI);
use DE_EPAGES::Database::API::Config qw(DatabaseConnectionData);
use DE_EPAGES::Database::API::Connection qw(RunOnStore GetCurrentDBHandle);
use DE_EPAGES::Database::API::DBInterface qw(ProcessSqlGo);
use DE_EPAGES::ShopMigration::API::ShortURL qw(ExistsRedirect);
use File::Temp qw(tempdir);
use Test::Exception;
use Test::More;
use Test::Differences qw(context_diff eq_or_diff);

my $TmpDir = tempdir();

if ($ENV{SKIP_IN_PR_TESTS}) {
    plan skip_all => 'Skipping it because SKIP_IN_PR_TESTS == 1';
}

sub test {
    context_diff();
    testExportWithCategoriesExportImportBase2Prep();
    testExportWithCategoriesExportImportPrep2Base();
    testImportWithCategoriesExportImportBase2Prep();
    testImportWithCategoriesExportImportPrep2Base();
    done_testing();
}

my %TestData = (
    TestShop      => undef,
    TestShopAlias => 'TestShop',
    ImportPath    => '/Test/Shops/TestShop',
);
my $SKIP_EPAGESUI_TESTS = 0;
use constant FILE_LINES_BASE => 11;
use constant FILE_LINES_PREP => 7;
use constant IMPORT_CATEGORIES_PREP => 2;
use constant IMPORT_CATEGORIES_BASE => 2;
use constant FILENAME => 'categories.json';
use constant FILE_DIR => [ qw(ShopData.d export TestShop) ];

sub testExportWithCategoriesExportImportBase2Prep {
    my $Shop = $TestData{TestShop};

    note('test CategoriesExportDriver (Base2Prep)');
    isnt($Shop, undef, 'Testshop exists');
    is(scalar @{$Shop->get('Languages')}, 1, 'Testshop has now one Language');
    is(scalar @{$Shop->get('SiteLocales')}, 1, 'Testshop has one SiteLocale');
    ok(!$Shop->get('IsMigrationPrepShop'), 'Testshop has IsMigrationPrepShop=0');

    my $Exporter = ExportCategoriesJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);

    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, FILE_LINES_BASE, 'file lines count');

    ok($Exporter->fileName =~ /${\FILENAME}/, "export file name matches ${\FILENAME}");

    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'base2prep_' . FILENAME), { encoding => 'utf8' });
    compareCategoriesHashes($sFileContent, $sExpected);

    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');

    _addLanguagesToShop($Shop);
    is(scalar @{$Shop->get('Languages')}, 3, 'Testshop has now three Languages');
    is(scalar @{$Shop->get('SiteLocales')}, 3, 'Testshop has now three SiteLocales');

    $Exporter = ExportCategoriesJSON($Shop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);

    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, FILE_LINES_BASE, 'file lines count');

    my $aContentLines = GetFileContentLines($Exporter->fileName, { encoding => 'utf8' });
    my $hValues = DecodeJSON($aContentLines->[0]);
    ok(exists $hValues->{LocalizedAttributes}{ru}, 'export file has localized attributes for language ru');
    ok(exists $hValues->{LocalizedAttributes}{de}, 'export file has localized attributes for language de');

    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub testExportWithCategoriesExportImportPrep2Base {
    my $PrepShop = $TestData{PrepShop};

    note('test CategoriesExportDriver (Prep2Base)');
    isnt($PrepShop, undef, 'PrepShop exists');
    is(scalar @{$PrepShop->get('Languages')}, 1, 'Testshop has now one Language');
    is(scalar @{$PrepShop->get('SiteLocales')}, 1, 'Testshop has one SiteLocale');
    ok($PrepShop->get('IsMigrationPrepShop'), 'PrepShop has IsMigrationPrepShop=1');

    my $Exporter = ExportCategoriesJSON($PrepShop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists ' . $Exporter->fileName);

    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, FILE_LINES_PREP, 'file lines count');

    ok($Exporter->fileName =~ /${\FILENAME}/, "export file name matches ${\FILENAME}");

    my $sFileContent = GetFileContent($Exporter->fileName, { encoding => 'utf8' });
    my $sExpected = GetFileContent(catfile(@{FILE_DIR()}, 'prep2base_' . FILENAME), { encoding => 'utf8' });
    compareCategoriesHashes($sFileContent, $sExpected);

    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');

    _addLanguagesToShop($PrepShop);
    is(scalar @{$PrepShop->get('Languages')}, 3, 'PrepShop has now three Languages');
    is(scalar @{$PrepShop->get('SiteLocales')}, 3, 'PrepShop has now three SiteLocales');

    $Exporter = ExportCategoriesJSON($PrepShop, $TmpDir);
    ok(ExistsFile($Exporter->fileName), 'export file exists: ' . $Exporter->fileName);

    is(scalar @{GetFileContentLines($Exporter->fileName, { encoding => 'utf8' })}, FILE_LINES_PREP, 'file lines count');

    my $aContentLines = GetFileContentLines($Exporter->fileName, { encoding => 'utf8' });
    my $hValues = DecodeJSON($aContentLines->[0]);
    ok(exists $hValues->{LocalizedAttributes}{ru}, 'export file has localized attributes for language ru');
    ok(exists $hValues->{LocalizedAttributes}{de}, 'export file has localized attributes for language de');

    DeleteFile($Exporter->fileName());
    ok(!ExistsFile($Exporter->fileName()), 'export file cleaned up');
}

sub compareCategoriesHashes {
    my ($CurrentContent, $ExpectedContent) = @_;

    # Removes all unknown (not installed language elements) from $Expected otherwise
    # we get a diff, as it contains all enabled languages after installing all langpacks
    my $aCurrent = DecodeJSONArray($$CurrentContent);
    my $aExpected = DecodeJSONArray($$ExpectedContent);
    my @KeepLanguages = ();
    foreach my $Code2 (map {$_->{'Code2'}} @{LoadRootObject()->get('Languages')}) {
        push @KeepLanguages, $Code2;
    }
    foreach my $Line (@$aExpected) {
        foreach my $Lang (keys %{$Line->{'LocalizedAttributes'}}) {
            next if ElementOf($Lang, @KeepLanguages);
            delete $Line->{'LocalizedAttributes'}{$Lang};
        }
    }
    eq_or_diff($aCurrent, $aExpected, 'file content matches');
}

sub testImportWithCategoriesExportImportBase2Prep {
    my $ExpectedValues = {
        Categories      => {
            Attributes          => {
                'IsVisible'           => 1,
                'Position'            => '10',
                'Parent'              => '.',
                'VisibleInNavigation' => 0,
            },
            LocalizedAttributes => {
                'en' => {
                    'Name'     => 'Home page',
                    SEOQuality => 0,
                }
            },
        },
        Tents           => {
            'Attributes'          => {
                'IsVisible'           => '1',
                'Position'            => '20',
                'Parent'              => 'Categories',
                'VisibleInNavigation' => 0
            },
            'LocalizedAttributes' => {
                'en' => {
                    'Name'     => 'Tents',
                    SEOQuality => 0,
                }
            }
        },
        IndividualTents => {
            'Attributes'          => {
                'IsVisible'           => '1',
                'Position'            => '30',
                'Parent'              => 'Categories/Tents',
                'VisibleInNavigation' => 0
            },
            'LocalizedAttributes' => {
                'en' => {
                    'Name'     => 'Individual tents',
                    SEOQuality => 0,
                }
            }
        }
    };

    my $PrepShop = $TestData{PrepShop};
    my $DirName = catdir('ShopData.d', 'import');
    note('test CategoriesImportHandler (Base2Prep)');
    isnt($PrepShop, undef, 'Prepshop exists');

    $PrepShop->folder('Categories')->deleteChildren;
    my @CurrentCategories;
    @CurrentCategories = map {$_->alias} @{$PrepShop->get('Categories')->get('RecursiveSubCategories')};
    is(scalar @CurrentCategories, 0, 'no categories on PrepShop');

    my $Importer = ImportCategoriesJSON($PrepShop, $DirName);
    @CurrentCategories = map {$_->alias} @{$PrepShop->get('Categories')->get('RecursiveSubCategories')};
    is(scalar @CurrentCategories, IMPORT_CATEGORIES_PREP, 'imported categories count');
    ok($Importer->fileName() =~ /TestShop.base2prep_categories[.]json/, 'ImportFileName was constructed properly');

    foreach my $Alias (keys %$ExpectedValues) {
        my $ParentPath = $ExpectedValues->{$Alias}{Attributes}{Parent};
        is(ExistsObjectByPath($ParentPath, $PrepShop->id), 1, sprintf('Parent %s for category %s exists', $ParentPath, $Alias));

        my $Parent = LoadObjectByPath($ParentPath, $PrepShop->id);
        is($Parent->existsChild($Alias), 1, sprintf('Category %s exists', $Alias));

        my $Category = $Parent->child($Alias);
        foreach my $Attribute (keys %{$ExpectedValues->{$Alias}{Attributes}}) {
            next if ($Attribute eq 'Parent');
            is($Category->get($Attribute), $ExpectedValues->{$Alias}{Attributes}{$Attribute}, "attribute $Attribute matches")
        }
        foreach my $Attribute (@{CATEGORIES_ATTRIBUTES_FOR_RESET()}) {
            is($Category->get($Attribute), undef, "attribute $Attribute matches (reset)")
        }
        foreach my $LanguageCode (keys %{$ExpectedValues->{$Alias}{LocalizedAttributes}}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (keys %{$ExpectedValues->{$Alias}{LocalizedAttributes}{$LanguageCode}}) {
                is($Category->get($Attribute, $LanguageID),
                    $ExpectedValues->{$Alias}{LocalizedAttributes}{$LanguageCode}{$Attribute},
                    "localized attribute $Attribute for $LanguageCode matches");
            }
            if ($Category->parent->id != $PrepShop->id) {
                isnt($Category->get('URI', $LanguageID), undef,
                    "localized attribute URI for $LanguageCode was set");
            }
        }
        foreach my $LanguageCode (map {$_->{'Code2'}} @{LoadRootObject()->get('Languages')}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (@{CATEGORIES_LOCALIZEDATTRIBUTES_FOR_RESET()}) {
                next if $Attribute eq 'SEOQuality';
                is($Category->get($Attribute, $LanguageID), undef,
                    "localized attribute $Attribute for $LanguageCode matches (reset)");
            }
        }
    }

    my $BadFileContent = '{"Attributes":{"Parent":".","Alias":"BadCategory"},"LocalizedAttributes":{"en":{"Name":"!!!"}}}';
    my $Dir = tempdir(CLEANUP => 1);
    my $FileName = catfile($Dir, $TestData{TestShopAlias}, 'base2prep_' . FILENAME);
    WriteFile($FileName, \$BadFileContent, { create_directory => 1 });
    throws_ok(sub {$Importer = ImportCategoriesJSON($PrepShop, $Dir)}, qr/NAME_NOT_VALID/, 'Invalid Name field throws error');
}

sub testImportWithCategoriesExportImportPrep2Base {
    my $Shop = $TestData{TestShop};
    TransferEpagesUI($Shop);
    my $ExpectedValues = {
        Categories => {
            Attributes          => {
                'IsVisible'           => 1,
                'Position'            => '10',
                'Parent'              => '.',
                'GUID'                => '99999999-0000-0000-0000-000000000001',
                'VisibleInNavigation' => 0,
            },
            LocalizedAttributes => {
                'en' => {
                    'Name'     => 'Home page',
                    SEOQuality => 0,
                }
            },
        },
        Shoes      => {
            'Attributes'          => {
                'IsVisible'           => '1',
                'Position'            => '20',
                'Parent'              => 'Categories',
                'GUID'                => '99999999-0000-0000-0000-000000000002',
                'VisibleInNavigation' => 0
            },
            'LocalizedAttributes' => {
                'en' => {
                    'Name'     => 'Shoes',
                    SEOQuality => 0,
                }
            }
        },
        ShoeInfo   => {
            'Attributes'          => {
                'IsVisible'           => '1',
                'Position'            => '30',
                'Parent'              => 'Categories/Shoes',
                'GUID'                => '99999999-0000-0000-0000-000000000003',
                'VisibleInNavigation' => 0
            },
            'LocalizedAttributes' => {
                'en' => {
                    'Name'     => 'How to Care for Your Shoes',
                    SEOQuality => 0,
                }
            }
        }
    };
    my $DirName = catdir('ShopData.d', 'import');
    note('test CategoriesImportHandler (Prep2Base)');
    isnt($Shop, undef, 'TestShop exists');

    $Shop->folder('Categories')->deleteChildren;
    my @CurrentCategories;
    @CurrentCategories = map {$_->alias} @{$Shop->get('Categories')->get('RecursiveSubCategories')};
    is(scalar @CurrentCategories, 0, 'no categories on Shop');

    my %GuidForPageID = (
        '00000000-0000-0000-0000-990000000001' => { Old => '99999999-0000-0000-0000-000000000001' },
        '00000000-0000-0000-0000-990000000002' => { Old => '99999999-0000-0000-0000-000000000002' },
        '00000000-0000-0000-0000-990000000003' => { Old => '99999999-0000-0000-0000-000000000003' },
    );
    my %PageIDForGUID;
    while (my ($key, $value) = each %GuidForPageID) {
        $PageIDForGUID{$value->{Old}} = $key;
    }
    for my $PageID (sort keys %GuidForPageID) {
        my $GUID = _getGUIDforPageID($PageID);
        unless ($SKIP_EPAGESUI_TESTS) {
            is($GUID, $GuidForPageID{$PageID}{Old}, sprintf('PageID %s has old GUID %s epages-ui table pages', $PageID, $GUID));
        }
    }

    my %WantedRedirects = (
        de_DE => [ qw(Shoes/de Shoes/de/ShoeInfo) ],
        en_GB => [ qw(Shoes Shoes/How-to-Care-for-Your-Shoes) ],
    );
    foreach my $Locale (keys %WantedRedirects) {
        foreach my $Redirect (@{$WantedRedirects{$Locale}}) {
            is(ExistsRedirect($Shop, $Redirect), 0,
                sprintf('No redirect in epages-ui yet for %s', $Redirect));
        }
    }
    my $Importer = ImportCategoriesJSON($Shop, $DirName);
    @CurrentCategories = map {$_->alias} @{$Shop->get('Categories')->get('RecursiveSubCategories')};
    is(scalar @CurrentCategories, IMPORT_CATEGORIES_BASE, 'imported categories count');
    ok($Importer->fileName() =~ /TestShop.prep2base_categories[.]json/, 'ImportFileName was constructed properly');

    foreach my $Locale (keys %WantedRedirects) {
        foreach my $Redirect (@{$WantedRedirects{$Locale}}) {
            is(ExistsRedirect($Shop, $Redirect), 1,
                sprintf('Redirect for %s added', $Redirect));
        }
    }

    foreach my $Alias (keys %$ExpectedValues) {
        my $ParentPath = $ExpectedValues->{$Alias}{Attributes}{Parent};
        is(ExistsObjectByPath($ParentPath, $Shop->id), 1, sprintf('Parent %s for category %s exists', $ParentPath, $Alias));

        my $Parent = LoadObjectByPath($ParentPath, $Shop->id);
        is($Parent->existsChild($Alias), 1, sprintf('Category %s exists', $Alias));

        my $Category = $Parent->child($Alias);
        foreach my $Attribute (keys %{$ExpectedValues->{$Alias}{Attributes}}) {
            next if ($Attribute eq 'Parent');
            if ($Attribute eq 'GUID') {
                my $OldGUID = $ExpectedValues->{$Alias}{Attributes}{$Attribute};
                my $PageID = $PageIDForGUID{$OldGUID};
                my $NewGUID = $Category->get($Attribute);
                $GuidForPageID{$PageID}{New} = $NewGUID;
                isnt($NewGUID, $OldGUID, "attribute $Attribute has changed to $NewGUID")
            } else {
                is($Category->get($Attribute), $ExpectedValues->{$Alias}{Attributes}{$Attribute}, "attribute $Attribute matches")
            }
        }
        foreach my $Attribute (@{CATEGORIES_ATTRIBUTES_FOR_RESET()}) {
            is($Category->get($Attribute), undef, "attribute $Attribute matches (reset)")
        }
        foreach my $LanguageCode (keys %{$ExpectedValues->{$Alias}{LocalizedAttributes}}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (keys %{$ExpectedValues->{$Alias}{LocalizedAttributes}{$LanguageCode}}) {
                is($Category->get($Attribute, $LanguageID),
                    $ExpectedValues->{$Alias}{LocalizedAttributes}{$LanguageCode}{$Attribute},
                    "localized attribute $Attribute for $LanguageCode matches");
            }
            if ($Category->parent->id != $Shop->id) {
                isnt($Category->get('URI', $LanguageID), undef,
                    "localized attribute URI for $LanguageCode was set");
            }
        }
        foreach my $LanguageCode (map {$_->{'Code2'}} @{LoadRootObject()->get('Languages')}) {
            my $LanguageID = GetPKeyLanguageByCode($LanguageCode);
            foreach my $Attribute (@{CATEGORIES_LOCALIZEDATTRIBUTES_FOR_RESET()}) {
                next if $Attribute eq 'SEOQuality';
                is($Category->get($Attribute, $LanguageID), undef,
                    "localized attribute $Attribute for $LanguageCode matches (reset)");
            }
        }
    }

    for my $PageID (sort keys %GuidForPageID) {
        my $GUID = _getGUIDforPageID($PageID);
        unless ($SKIP_EPAGESUI_TESTS) {
            is($GUID, $GuidForPageID{$PageID}{New}, sprintf('PageID %s has new GUID %s epages-ui table pages', $PageID, $GUID));
        }
    }

    my $BadFileContent = '{"Attributes":{"Parent":".","Alias":"BadCategory"},"LocalizedAttributes":{"en":{"Name":"!!!"}}}';
    my $Dir = tempdir(CLEANUP => 1);
    my $FileName = catfile($Dir, $TestData{TestShopAlias}, 'prep2base_' . FILENAME);
    WriteFile($FileName, \$BadFileContent, { create_directory => 1 });
    throws_ok(sub {$Importer = ImportCategoriesJSON($Shop, $Dir)}, qr/NAME_NOT_VALID/, 'Invalid Name field throws error');
}

sub _addLanguagesToShop() {
    my ($Shop) = @_;
    foreach my $Lang (qw(de en ru)) {
        my $LanguageID = GetPKeyLanguageByCode($Lang);
        $Shop->addLanguage($LanguageID) unless $Shop->existsLanguage($LanguageID);
    }
    foreach my $Locale (qw(ru_RU de_DE en_GB)) {
        $Shop->addLocale($Locale) unless $Shop->existsLocale($Locale);
    }
}

sub _getGUIDforPageID() {
    my ($PageID) = @_;
    my $Store = LoadRootObject()->get('StoreName');
    my $UIStore = DatabaseConnectionData($Store)->{InstanceUnityUI};
    my $GUID = undef;
    if (!defined $UIStore || !$UIStore) {
        $SKIP_EPAGESUI_TESTS = 1;
    } else {
        RunOnStore(
            Store => $UIStore,
            Sub   => sub {
                my $dbi = GetCurrentDBHandle();
                my $dbResult = $dbi->execute('SELECT guid FROM pages where id=?', [ $PageID ], [ 'String' ]);
                $GUID = $dbResult->[0][0] if (scalar @$dbResult);
            }
        );
    }
    return $GUID;
}

sub _createFakePagesEntries() {
    _doSQLFileOnUI(catfile(qw(ImportFiles mysql pages.sql)));
    _updateFakeEntriesForCurrentShop();
}

sub _cleanupFakePagesEntries() {
    _doSQLFileOnUI(catfile(qw(ImportFiles mysql delete_pages.sql)));
}

sub _updateFakeEntriesForCurrentShop {
    my $Store = LoadRootObject()->get('StoreName');
    my $UIStore = DatabaseConnectionData($Store)->{InstanceUnityUI};
    if (!defined $UIStore || !$UIStore) {
        $SKIP_EPAGESUI_TESTS = 1;
    } else {
        RunOnStore(
            Store => $UIStore,
            Sub   => sub {
                my $dbi = GetCurrentDBHandle();
                my $aaRows = $dbi->execute(<<SQL);
SELECT id FROM shops WHERE shopGateID = (
    SELECT id FROM shopGates WHERE slug='testshop'
)
SQL
                my $OldShopID = '00000001-0001-0001-0001-000000000001';
                my $NewShopID = $aaRows->[0][0];
                $dbi->execute('update pages set shopId=? where shopid=?', [ $NewShopID, $OldShopID ]);
            }
        );
    };
}

sub _doSQLFileOnUI() {
    my ($SQLFile) = @_;
    my $Store = LoadRootObject()->get('StoreName');
    my $UIStore = DatabaseConnectionData($Store)->{InstanceUnityUI};
    if (!defined $UIStore || !$UIStore) {
        $SKIP_EPAGESUI_TESTS = 1;
    } else {
        RunOnStore(
            Store => $UIStore,
            Sub   => sub {
                my $SQLBlock = GetFileContent($SQLFile);
                ProcessSqlGo($$SQLBlock);
            },
        );
    }
}

# We need to create a fake ShopTypeRef UnityDE + ECom5000 in case they don't exists
# otherwise the Mapper won't work
sub init {
    my $self = shift;

    $self->SUPER::init;
    my $ShopAlias = $TestData{TestShopAlias};
    my $System = LoadRootObject();
    my $Shop;
    my $TestFolder = $System->folder('Test')->folder('Shops');
    my $ShopTypeRefFolder = $System->folder('ShopTypeRefs');
    my $Class = LoadClassByAlias('ShopTypeRef');
    $Class->insertObject({ Alias => 'UnityDE', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('UnityDE'));
    $Class->insertObject({ Alias => 'ECom5000', 'Parent' => $ShopTypeRefFolder })
        unless ($ShopTypeRefFolder->existsChild('ECom5000'));
    if ($TestFolder->existsChild($ShopAlias)) {
        $Shop = $TestFolder->child($ShopAlias);
    } else {
        note(sprintf('Creating %s', $ShopAlias));
        $Shop = $TestFolder->insertChild('Shop', {
            Alias                   => $ShopAlias,
            ShopTypeRef             => $ShopTypeRefFolder->child('ECom5000'),
            WebServerScriptNamePart => $ShopAlias,
        });
        SetShopFeatureValues($Shop, [
            { Alias => 'Regions', MaxValue => '2' },
            { Alias => 'Languages', MaxValue => '15' },
            { Alias => 'Pages', MaxValue => '100' },
            { Alias => 'Subcategories', MaxValue => '100' },
        ]);
    }
    note('Creating PrepShop') if (!ExistsPrepShop($Shop));
    $TestData{TestShop} = $Shop;
    $TestData{PrepShop} = ExistsPrepShop($Shop) ? $Shop->get('MigrationPrepShop') : CreatePrepShop($Shop);
    SetShopFeatureValues($TestData{PrepShop}, [ { Alias => 'Languages', MaxValue => '15' } ]);
    foreach my $JSONFile (@{GetFileNames('ShopData.d', { 'recursive' => 1 })}) {
        Text2Unix($JSONFile) if ($^O eq 'linux');
    }
    ImportObjects('ImportFiles/CategoriesBase.xml', $TestData{TestShop}->pathString());
    ImportObjects('ImportFiles/CategoriesPrep.xml', $TestData{PrepShop}->pathString());
    _createFakePagesEntries();
    return;
}

sub cleanup {
    my $self = shift;

    my $ShopAlias = $TestData{TestShopAlias};
    my $TestFolder = LoadRootObject()->folder('Test')->folder('Shops');
    _cleanupFakePagesEntries();
    if ($TestFolder->existsChild($ShopAlias)) {
        my $Shop = $TestFolder->child($ShopAlias);
        my $PrepShop = $Shop->get('MigrationPrepShop');
        if (defined $PrepShop) {
            note('Cleaning up PrepShop');
            $PrepShop->delete();
        }
        note(sprintf('Cleaning up %s', $ShopAlias));
        $Shop->delete();
        LoadRootObject()->folder('Test')->delete();
    }
    DeleteDirectory($TmpDir, { 'recursive' => 1 });
    return;
}

__PACKAGE__->runScript();

1;
