#========================================================================================
# package      DE_EPAGES::WebInterface::Servlet::BaseServlet
# state        public
#----------------------------------------------------------------------------------------
# description  Provides basic functionality for handling of HTTP requests.
#========================================================================================
package DE_EPAGES::WebInterface::Servlet::BaseServlet;

use strict;
use HTTP::Status qw (HTTP_MOVED_PERMANENTLY);
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::TestParameter qw( TestString );
use DE_EPAGES::WebInterface::API::ApplicationServer qw( GetAppServer );
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::PerlTools qw( GetCallerString );
#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet = new DE_EPAGES::Presentation::Servlet::Permission;
# example      my $Servlet = DE_EPAGES::WebInterface::Servlet::BaseServlet->new;
#----------------------------------------------------------------------------------------
# description  Create a new servlet object.
#----------------------------------------------------------------------------------------
# return       $Servlet | blessed object | object
#========================================================================================
sub new {
    my $class = shift;

    return bless {
        Request => undef,
        Response => undef,
    }, $class;
}

#========================================================================================
# function     init
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->init( $hParams );
#----------------------------------------------------------------------------------------
# description  Initializes the servlet for a new request with parameters from the
#               Servlet.conf configuration file.
#----------------------------------------------------------------------------------------
# input        $hParams | hash with servlet parameters | ref.hash.string
#========================================================================================
sub init {
    my $self = shift;
    my ($hParams) = @_;

    return;
}

#========================================================================================
# function     request
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Request = $Servlet->request;
# example      my $Request = $Servlet->request;
#----------------------------------------------------------------------------------------
# description  Return the current request.
#               This function must be called during the <function handleRequest> method,
#               otherwise it returns undef.
#----------------------------------------------------------------------------------------
# return       $Request | HTTP::Request object | object
#========================================================================================
sub request {
    my $self = shift;
    return $self->{'Request'};
}

#========================================================================================
# function     response
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Response = $Servlet->response;
# example      my $Response = $Servlet->response;
#----------------------------------------------------------------------------------------
# description  Return the current response.
#               This function must be called during the <function handleRequest> method,
#               otherwise it returns undef.
#----------------------------------------------------------------------------------------
# return       $Response | HTTP::Response object | object
#========================================================================================
sub response {
    my $self = shift;
    return $self->{'Response'};
}

#========================================================================================
# function     serverVariables
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ServerVariables = $Servlet->serverVariables;
# syntax       $ServerVariable = $Servlet->serverVariables($Name);
#----------------------------------------------------------------------------------------
# description  Without parameter returns the function cookies which was served by the
#               request. On first run on servlet the form will be encoded.<br>
#               With one parameter (name of server variable) the value of this key will be
#               returned.
#----------------------------------------------------------------------------------------
# input        $Name | name of cookie (optional) | string
# return       $ServerVariables | key value pairs (optional) | ref.hash
# return       $ServerVariable  | value of given name (optional) | string
#========================================================================================
sub serverVariables {
    my $self = shift;
    my (@Params) = @_;

    if (not defined $self->{'ServerVariables'}) {
        my $AppServer = GetAppServer;
        $self->{'ServerVariables'} = defined $AppServer ? $AppServer->serverVariables : {};
    }
    if (scalar @Params == 0) { # get hash
        return $self->{'ServerVariables'};
    } elsif (scalar @Params == 1) { # get
        return $self->{'ServerVariables'}->{$Params[0]};
    } else {
        Error('TOO_MANY_PARAMETER');
    }
    return $self->{'ServerVariables'};
}

#========================================================================================
# function     requestHeaders
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hHeaders = $Servlet->requestHeaders;
# syntax       $Value= $Servlet->requestHeaders($Name);
#----------------------------------------------------------------------------------------
# description  Without parameter returns all HTTP request headers as hash ref.<br />
#               With one parameter returns the value of one request header.
#----------------------------------------------------------------------------------------
# input        $Name | (optional) name of the HTTP header | string
# return       $hHeaders | all HTTP headers | ref.hash
# return       $Value  | value of one HTTP header | string
#========================================================================================
sub requestHeaders {
    my $self = shift;
    my ($Name) = @_;

    if ( 0 == scalar @_ ) { # get hash
        return $self->request->headers;
    }
    elsif ( 1 == scalar @_ ) { # get
        TestString('Name', $Name);
        return $self->request->header($Name);
    }
    else {
        Error('TOO_MANY_PARAMETER');
    }
}

#========================================================================================
# function     redirect
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->redirect( $Location );
# syntax       $Servlet->redirect( $Location, $StatusCode );
#----------------------------------------------------------------------------------------
# description  Redirects the browser to another URL.
#----------------------------------------------------------------------------------------
# input        $Location | target URL | string
# input        $StatusCode | (optional, default: 301) HTTP status code | int
#========================================================================================
sub redirect {
    my $self = shift;
    my( $Location, $StatusCode ) = @_;

    GetLog()->debug('redirect called ' . GetCallerString());
    TestString('Location', $Location);
    TestString('StatusCode', $StatusCode) if defined $StatusCode;
    $StatusCode = HTTP_MOVED_PERMANENTLY unless defined $StatusCode;

    my $Response = $self->response;
    $Response->code( $StatusCode );
    $Response->header( 'Location', $Location );
    $Response->content_type( "text/plain; charset=utf-8" );

    my $Content = "Moved to: $Location";
    utf8::encode( $Content );
    $Response->content( $Content );

    return;
}

#========================================================================================
# function     storeName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $StoreName = $Servlet->storeName;
#----------------------------------------------------------------------------------------
# description  Returns the store name from the X-ePages-Store header.
#               Returns undef if the header does not exist.
# return       $StoreName | name of the store | string
#========================================================================================
sub storeName {
    my $self = shift;
    return $self->requestHeaders('X-ePages-Store');
}

#========================================================================================
# function     handle
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->handle($Request, $Response);
#----------------------------------------------------------------------------------------
# description  Initializes <function request> and <function response> and calls
#               <function handleRequest>.
#----------------------------------------------------------------------------------------
# input        $Request | HTTP::Request object | object
# input        $Response | HTTP::Response object | object
#========================================================================================
sub handle {
    my $self = shift;
    my ($Request, $Response) = @_;

    local $self->{'Request'}  = $Request;
    local $self->{'Response'} = $Response;

    $self->handleRequest;
    return;
}

#========================================================================================
# function     handleRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->handleRequest;
#----------------------------------------------------------------------------------------
# description  Main function for handling the request.
#               This method must be overloaded by any sub class.
#               The overloaded method should use <function request> and
#               <function response> to access the current request and response objects.
#========================================================================================
sub handleRequest {
    die 'handleRequest is not implemented';
}

1;
